# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2022, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
The module is fuzzing UEFI Variable interface.

The module is using UEFI SetVariable interface to write new UEFI variables
to SPI flash NVRAM with randomized name/attributes/GUID/data/size.

Usage:
    ``chipsec_main -m tools.uefi.uefivar_fuzz [-a <options>]``

Options:

    ``[-a <test>,<iterations>,<seed>,<test_case>]``

        - ``test``       : UEFI variable interface to fuzz (all, name, guid, attrib, data, size)
        - ``iterations`` : Number of tests to perform (default = 1000)
        - ``seed``       : RNG seed to use
        - ``test_case``  : Test case # to skip to (combined with seed, can be used to skip to failing test)

    All module arguments are optional

Examples::
    >>> chipsec_main.py -m tools.uefi.uefivar_fuzz
    >>> chipsec_main.py -m tools.uefi.uefivar_fuzz -a all,100000
    >>> chipsec_main.py -m tools.uefi.uefivar_fuzz -a data,1000,123456789
    >>> chipsec_main.py -m tools.uefi.uefivar_fuzz -a name,1,123456789,94

.. note::
    - This module returns a WARNING by default to indicate that a manual review is needed.
    - Writes may generate 'ERROR's, this can be expected behavior if the environment rejects them.

.. warning::
    - This module modifies contents of non-volatile SPI flash memory (UEFI Variable NVRAM).
    - This may render system UNBOOTABLE if firmware doesn't properly handle variable update/delete operations.

.. important::
    - Evaluate the platform for expected behavior to determine PASS/FAIL.
    - Behavior can include platform stability and retaining protections.

"""

import random
from time import time
from uuid import uuid4, UUID
import struct

from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult
from chipsec.library.file import write_file
from chipsec.hal.uefi import UEFI
from chipsec.library.defines import bytestostring

from chipsec.fuzzing import primitives as prim

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'TOOLS', 'UEFI', 'UEFIVAR_FUZZ']


class uefivar_fuzz(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)
        self._uefi = UEFI(self.cs)

    def is_supported(self):
        supported = self.cs.helper.EFI_supported()
        if not supported:
            self.logger.log_important("OS does not support UEFI Runtime API.  Skipping module.")
        return supported

    def rnd(self, n=1):
        rnum = b''
        for j in range(n):
            rnum += struct.pack("B", random.randint(0, 255))
        return rnum

    def usage(self):
        self.logger.log(__doc__.translate({ord('`'): None}))
        return True

    def run(self, module_argv):
        self.logger.start_test("Fuzz UEFI Variable Interface")

        self.logger.log_warning("Are you sure you want to continue fuzzing UEFI variable interface?")
        s = input("Type 'yes' to continue > ")
        if s.lower() not in ['yes', 'y']:
            return

        # Default options
        _NAME = 'FuzzerVarName'
        _GUID = UUID('414C4694-F4CF-0525-69AF-C99C8596530F')
        _ATTRIB = 0x07
        _SIZE = 0x08
        _DATA = struct.pack("B", 0x41) * _SIZE

        ITERATIONS = 1000
        SEED = int(time())
        CASE = 1
        BOUND_STR = 255  # tested value that can be increased or decreased to fit the limit bounds
        BOUND_INT = 1000

        FUZZ_NAME = True
        FUZZ_GUID = True
        FUZZ_ATTRIB = True
        FUZZ_DATA = True
        FUZZ_SIZE = True

        # Init fuzzing primitives
        name_prim = prim.string(value=_NAME, max_len=BOUND_STR)
        attrib_prim = prim.dword(value=_ATTRIB)  # i think the attrib field is 4 bytes large?
        data_prim = prim.random_data(value=_DATA, min_length=0, max_length=BOUND_INT)

        help_text = False

        if len(module_argv):
            fz_cli = module_argv[0].lower()
            if 'all' != fz_cli:
                FUZZ_NAME = False
                FUZZ_GUID = False
                FUZZ_ATTRIB = False
                FUZZ_DATA = False
                FUZZ_SIZE = False

                if 'name' == fz_cli:
                    FUZZ_NAME = True
                elif 'guid' == fz_cli:
                    FUZZ_GUID = True
                elif 'attrib' == fz_cli:
                    FUZZ_ATTRIB = True
                elif 'data' == fz_cli:
                    FUZZ_DATA = True
                elif 'size' == fz_cli:
                    FUZZ_SIZE = True
                else:
                    help_text = self.usage()

            if len(module_argv) > 1:
                if module_argv[1].isdigit():
                    ITERATIONS = int(module_argv[1])
                else:
                    help_text = self.usage()

            if len(module_argv) > 2:
                if module_argv[2].isdigit():
                    SEED = int(module_argv[2])
                else:
                    help_text = self.usage()

            if len(module_argv) > 3:
                if module_argv[3].isdigit():
                    CASE = int(module_argv[3])
                else:
                    help_text = self.usage()

        if not help_text:
            random.seed(SEED)
            write_file('SEED.txt', str(SEED))

            if not len(module_argv):
                fz_cli = 'all'
            self.logger.log(f'Test      : {fz_cli}')
            self.logger.log(f'Iterations: {ITERATIONS:d}')
            self.logger.log(f'Seed      : {SEED:d}')
            self.logger.log(f'Test case : {CASE:d}')
            self.logger.log('')
            for count in range(1, ITERATIONS + CASE):
                if FUZZ_NAME:
                    _NAME = ''
                    if name_prim.mutate():
                        _NAME = name_prim.render()
                    else:
                        # if mutate() returns false, we need to reload the primitive
                        name_prim = prim.string(value=_NAME, max_len=BOUND_STR)
                        _NAME = name_prim.render()

                if FUZZ_GUID:
                    _GUID = uuid4()

                if FUZZ_ATTRIB:
                    if attrib_prim.mutate():
                        _ATTRIB = attrib_prim.render()
                    else:
                        attrib_prim = prim.dword(value=_ATTRIB)
                        _ATTRIB = attrib_prim.render()

                if FUZZ_DATA:
                    if data_prim.mutate():
                        _DATA = data_prim.render()
                    else:
                        data_prim = prim.random_data(value=_DATA, min_length=0, max_length=BOUND_INT)
                        data_prim.mutate()
                        _DATA = data_prim.render()

                if FUZZ_SIZE:
                    if _DATA:
                        _SIZE = random.randrange(len(_DATA))
                    else:
                        _SIZE = random.randrange(1024)

                if count < CASE:
                    continue

                self.logger.log(f'  Running test #{count:d}:')
                self.logger.flush()
                status = self._uefi.set_EFI_variable(bytestostring(_NAME), str(_GUID), _DATA, _SIZE, _ATTRIB)
                self.logger.log(status)
                status = self._uefi.delete_EFI_variable(bytestostring(_NAME), str(_GUID))
                self.logger.log(status)

        self.logger.log_warning('Fuzzing complete: platform is in an unknown state.')
        self.logger.log_important('Evaluate the platform for expected behavior to determine PASS/FAIL')
        self.logger.log_important('Behavior can include platform stability and retaining protections.')

        self.result.setStatusBit(self.result.status.VERIFY)
        return self.result.getReturnCode(ModuleResult.WARNING)



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2024, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
Generate a module ID using hashlib from the module's file name (no file extension).
Hash is truncated to 28 bits. For module names use full path as seen in example below.

Usage:
    ``chipsec_util id name <module name>``
    ``chipsec_util id hash <module hash>``

Examples:
    >>> chipsec_util.py id name chipsec.modules.common.remap
    >>> chipsec_util.py id hash 0x67eb58d
"""

from chipsec.library.returncode import get_module_ids_dictionary, generate_hash_id
from chipsec.command import BaseCommand, toLoad
from argparse import ArgumentParser


class ModuleIdCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.Nil

    def parse_arguments(self) -> None:
        parser = ArgumentParser(prog='chipsec_util id', usage=__doc__)
        subparsers = parser.add_subparsers()

        parser_read = subparsers.add_parser('name')
        parser_read.add_argument('module_name', type=str, help='Module name')
        parser_read.set_defaults(func=self.get_id_from_name)

        parser_write = subparsers.add_parser('hash')
        parser_write.add_argument('module_id', type=str, help='Module ID')
        parser_write.set_defaults(func=self.get_name_from_id)

        parser.parse_args(self.argv, namespace=self)

    def get_id_from_name(self) -> None:
        module_ids = get_module_ids_dictionary()
        if self.module_name in module_ids:
            module_id = module_ids[self.module_name]
        else:
            module_id = generate_hash_id(self.module_name)
        self.logger.log(f'Module ID is: {hex(module_id)}\n')

    def get_name_from_id(self) -> None:
        module_ids = get_module_ids_dictionary()
        try:
            module_name = list(module_ids.keys())[list(module_ids.values()).index(int(self.module_id, 16))]
            self.logger.log(f'Module name is: {module_name}\n')
        except IndexError:
            self.logger.log(f'Could not find {self.module_id}\n')


commands = {'id': ModuleIdCommand}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
>>> chipsec_util smbus read <device_addr> <start_offset> [size]
>>> chipsec_util smbus write <device_addr> <offset> <byte_val>

Examples:

>>> chipsec_util smbus read 0xA0 0x0 0x100
"""

from chipsec.command import BaseCommand, toLoad
from chipsec.library.logger import print_buffer_bytes
from chipsec.hal.smbus import SMBus
from argparse import ArgumentParser


class SMBusCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.All

    def parse_arguments(self) -> None:
        parser = ArgumentParser(prog='chipsec_util smbus', usage=__doc__)
        subparsers = parser.add_subparsers()
        parser_read = subparsers.add_parser('read')
        parser_read.add_argument('dev_addr', type=lambda x: int(x, 16), help='Start Address (hex)')
        parser_read.add_argument('start_off', type=lambda x: int(x, 16), help='Start Offset (hex)')
        parser_read.add_argument('size', type=lambda x: int(x, 16), default=None, nargs='?', help='Size [Default=Byte] (hex)')
        parser_read.set_defaults(func=self.smbus_read)

        parser_write = subparsers.add_parser('write')
        parser_write.add_argument('dev_addr', type=lambda x: int(x, 16), help='Start Address (hex)')
        parser_write.add_argument('off', type=lambda x: int(x, 16), help='Start Offset (hex)')
        parser_write.add_argument('val', type=lambda x: int(x, 16), help='Byte Value (hex)')
        parser_write.set_defaults(func=self.smbus_write)

        parser.parse_args(self.argv, namespace=self)

    def set_up(self) -> None:
        self._smbus = SMBus(self.cs)

    def smbus_read(self):
        if self.size is not None:
            buf = self._smbus.read_range(self.dev_addr, self.start_off, self.size)
            self.logger.log("[CHIPSEC] SMBus read: device 0x{:X} offset 0x{:X} size 0x{:X}".format(self.dev_addr, self.start_off, self.size))
            print_buffer_bytes(buf)
        else:
            val = self._smbus.read_byte(self.dev_addr, self.start_off)
            self.logger.log("[CHIPSEC] SMBus read: device 0x{:X} offset 0x{:X} = 0x{:X}".format(self.dev_addr, self.start_off, val))

    def smbus_write(self):
        self.logger.log("[CHIPSEC] SMBus write: device 0x{:X} offset 0x{:X} = 0x{:X}".format(self.dev_addr, self.off, self.val))
        self._smbus.write_byte(self.dev_addr, self.off, self.val)

    def run(self):
        if not self._smbus.is_SMBus_supported():
            self.logger.log("[CHIPSEC] SMBus controller is not supported")
            return
        self._smbus.display_SMBus_info()
        self.func()


commands = {'smbus': SMBusCommand}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2020, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
The configuration of the SPI controller, including protected ranges (PR0-PR4), is locked by HSFS[FLOCKDN] until reset.
If not locked, the controller configuration may be bypassed by reprogramming these registers.

This vulnerability (not setting FLOCKDN) is also checked by other tools, including  `flashrom <http://www.flashrom.org/>`_
and Copernicus by MITRE.

This module checks that the SPI Flash Controller configuration is locked.

Reference:
    - `flashrom <http://www.flashrom.org/>`_
    - `Copernicus: Question Your Assumptions about BIOS Security <http://www.mitre.org/capabilities/cybersecurity/overview/cybersecurity-blog/copernicus-question-your-assumptions-about>`_

Usage:
    ``chipsec_main -m common.spi_lock``

Examples:
    >>> chipsec_main.py -m common.spi_lock

Registers used:
    - FlashLockDown (control)
    - SpiWriteStatusDis (control)

"""

from chipsec.library.exceptions import CSReadError
from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult
from typing import List

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'SPI_LOCK']


class spi_lock(BaseModule):

    def __init__(self):
        super(spi_lock, self).__init__()

    def is_supported(self) -> bool:
        if self.cs.control.is_defined('FlashLockDown'):
            return True
        self.logger.log_important('FlashLockDown control not define for platform.  Skipping module.')
        return False

    def check_spi_lock(self) -> int:
        res = ModuleResult.PASSED
        reg_print = True
        if self.cs.control.is_defined('SpiWriteStatusDis'):
            wrsdis = self.cs.control.get('SpiWriteStatusDis', with_print=reg_print)
            if 1 == wrsdis:
                self.logger.log_good('SPI write status disable set.')
            else:
                res = ModuleResult.FAILED
                self.result.setStatusBit(self.result.status.ACCESS_RW)
                self.logger.log_bad('SPI write status disable not set.')
            reg_print = False

        flockdn = self.cs.control.get('FlashLockDown', with_print=reg_print)
        if 1 == flockdn:
            self.logger.log_good('SPI Flash Controller configuration is locked')
        else:
            res = ModuleResult.FAILED
            self.result.setStatusBit(self.result.status.LOCKS)
            self.logger.log_bad('SPI Flash Controller configuration is not locked')
        reg_print = False

        if res == ModuleResult.FAILED:
            self.logger.log_failed('SPI Flash Controller not locked correctly.')
        elif res == ModuleResult.PASSED:
            self.logger.log_passed('SPI Flash Controller locked correctly.')
        else:
            self.logger.log_warning('Unable to determine if SPI Flash Controller is locked correctly.')

        return self.result.getReturnCode(res)

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('SPI Flash Controller Configuration Locks')
        try:
            self.res = self.check_spi_lock()
        except CSReadError as err:
            self.logger.log_warning(f'Unable to read register: {err}')
            self.result.setStatusBit(self.result.status.VERIFY)
            self.res = self.result.getReturnCode(ModuleResult.WARNING)
        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Xen specific hypercall functionality
"""

from chipsec.modules.tools.vmm.xen.define import *
from chipsec.hal.vmm import *
from chipsec.module_common import *
from chipsec.modules.tools.vmm.common import *

SM_RANGE = {'masks': [0x00000000000000FF]}
MD_RANGE = {'masks': [0x000000000000FFFF]}
XL_RANGE = {'masks': [0xFFFFFFFFFFFFFFFF]}


class XenHypercall(BaseModuleHwAccess):
    def __init__(self):
        BaseModuleHwAccess.__init__(self)
        self.vmm = VMM(self.cs)
        self.vmm.init()
        self.hypervisor_present = False
        self.hypercalls = {}
        self.buff_va = 0
        self.buff_pa = 0
        (self.buff_va, self.buff_pa) = self.cs.mem.alloc_physical_mem(0x1000, 0xFFFFFFFFFFFFFFFF)
        if self.buff_pa == 0:
            raise Exception("[*] Could not allocate memory!")
        # define initial args for hypercall fuzzing
        self.hypercall_args = {
            MEMORY_OP: {'args': [xenmem_commands.keys(), self.buff_va]},
            SET_TIMER_OP: {'args': [XL_RANGE, XL_RANGE]},
            XEN_VERSION: {'args': [xen_version_commands.keys(), self.buff_va]},
            CONSOLE_IO: {'args': [console_io_commands.keys(), MD_RANGE, self.buff_va]},
            GRANT_TABLE_OP: {'args': [SM_RANGE, self.buff_va, XL_RANGE]},
            SCHED_OP: {'args': [schedop_commands.keys(), self.buff_va]},
            EVENT_CHANNEL_OP: {'args': [evtchop_commands.keys(), self.buff_va]},
            NMI_OP: {'args': [XL_RANGE, XL_RANGE]},
            HVM_OP: {'args': [hvmop_commands.keys(), self.buff_va]},
            TMEM_OP: {'args': [self.buff_va]},
            XENPMU_OP: {'args': [xenpmuop_commands.keys(), self.buff_va]},
            SYSCTL: {'args': [self.buff_va]},
            DOMCTL: {'args': [self.buff_va]},
            ARCH_1: {'args': [self.buff_va]},
        }

    def get_value(self, arg):
        if type(arg) in [dict]:
            value = random.choice(arg.get('masks')) & random.getrandbits(64)
        elif type(arg) in [list, tuple, range]:
            value = random.choice(arg)
        else:
            value = arg
        return value

    ##
    # hypercall
    ##
    def hypercall(self, args, size=0, data=''):
        data = data.ljust(4096, '\x00')[:4096]
        self.cs.mem.write_physical_mem(self.buff_pa, len(data), data)
        self.dbg(f'ARGS: {" ".join([f"{x:016X}" for x in args])}  DATA: {data[:32].hex()}')
        try:
            rax = self.vmm.hypercall64_five_args(*args)
            val = self.cs.mem.read_physical_mem(self.buff_pa, size) if size > 0 else ''
        except Exception as e:
            self.dbg(f'Exception on hypercall (0x{args[0]:08X}): {str(e)}')
            return {'exception': True, 'status': 0xFFFFFFFFFFFFFFFF, 'buffer': str(e)}
        return {'exception': False, 'status': rax, 'buffer': val}

    ##
    # xen_version
    ##
    def xen_version(self, cmd, size=0, data=''):
        return self.hypercall((XEN_VERSION, cmd, self.buff_va), size, data)

    ##
    # get_hypervisor_info
    ##
    def get_hypervisor_info(self):
        info = {}

        if not self.xen_version(XENVER_VERSION)['exception']:
            version = self.xen_version(XENVER_VERSION)
            extra_version = self.xen_version(XENVER_EXTRAVERSION, 16)
            compile_info = self.xen_version(XENVER_COMPILE_INFO, 140)
            capabilities = self.xen_version(XENVER_CAPABILITIES, 1024)
            changeset = self.xen_version(XENVER_CHANGESET, 64)
            platform_parameters = self.xen_version(XENVER_PLATFORM_PARAMETERS, 8)
            pagesize = self.xen_version(XENVER_PAGESIZE)
            guest_handle = self.xen_version(XENVER_GUEST_HANDLE)
            command_line = self.xen_version(XENVER_COMMANDLINE, 1024)

            info['extra_version'] = extra_version['buffer'].strip('\x00')
            info['xen_major'] = (version['status'] >> 16) & 0xFFFF
            info['xen_minor'] = version['status'] & 0xFFFF
            info['xen_version'] = f'{info["xen_major"]:d}.{info["xen_minor"]:d}{extra_version["buffer"]}'
            info['compiler'] = compile_info['buffer'][:64].strip('\x00')
            info['compile_by'] = compile_info['buffer'][64:80].strip('\x00')
            info['compile_domain'] = compile_info['buffer'][80:112].strip('\x00')
            info['compile_date'] = compile_info['buffer'][112:140].strip('\x00')
            info['capabilities'] = capabilities['buffer'].strip('\x00')
            info['changeset'] = changeset['buffer'].strip('\x00')
            info['platform_parameters'] = struct.unpack('<Q', platform_parameters['buffer'])[0]
            info['pagesize'] = pagesize['status']
            info['guest_handle'] = guest_handle['status']
            info['command_line'] = command_line['buffer'].strip('\x00')

            info['features'] = {}
            for i in range(0x100):
                feature = self.xen_version(XENVER_GET_FEATURES, 8, struct.pack('<LL', i, 0))
                if feature['exception'] == False:
                    values = struct.unpack('<LL', feature['buffer'])
                    info['features'][values[0]] = values[1]

        return info

    ##
    # print_hypervisor_info
    ##
    def print_hypervisor_info(self, info):
        features = ', '.join([f'F{k:d}={v:016X}' for k, v in info['features'].items() if v != 0])
        self.msg(f'XEN Hypervisor is present!')
        self.msg(f'          Version : {info["xen_version"]}')
        self.msg(f'         Compiler : {info["compiler"]}')
        self.msg(f'       Compile by : {info["compile_by"]}')
        self.msg(f'   Compile Domain : {info["compile_domain"]}')
        self.msg(f'     Compile Date : {info["compile_date"]}')
        self.msg(f'     Capabilities : {info["capabilities"]}')
        self.msg(f'       Change Set : {info["changeset"]}')
        self.msg(f'  Platform Params : {info["platform_parameters"]:016X}')
        self.msg(f'         Features : {features}')
        self.msg(f'        Page size : {info["pagesize"]:016X}')
        self.msg(f'     Guest Handle : {info["guest_handle"]:016X}')
        self.msg(f'     Command Line : {info["command_line"]}')

    def scan_hypercalls(self, vector_list):
        for vector in vector_list:
            args = self.hypercall_args.get(vector, {}).get('args', [])
            result = self.hypercall([vector] + [0 for a in args])
            if (result['exception'] == False) and (result['status'] != get_invalid_hypercall_code()):
                self.hypercalls[vector] = result
                self.add_initial_data(vector, result['buffer'], get_hypercall_status(result['status'], True))
        return

    def print_hypercall_status(self):
        self.msg('')
        self.msg('*** Hypervisor Hypercall Status Codes ***')
        for vector in sorted(self.hypercalls.keys()):
            data = self.hypercalls[vector]
            name = get_hypercall_name(vector)
            status = get_hypercall_status_extended(data['status'])
            self.msg(f'HYPERCALL {vector:04X}  {data["status"]:016X}  {status:45} \'{name}\'')
        return

    def fuzz_hypercall(self, code, iterations):
        rule = self.hypercall_args.get(code, {})
        if not rule:
            self.msg("WARNING: Fuzzing rule is not defined for this hypercall!")
        args = rule.get('args', [])
        self.msg(f'Fuzzing {get_hypercall_name(code, "Unknown")} (0x{code:02X}) hypercall')
        self.stats_reset()
        for it in range(iterations):
            data = list('\x00' * 32)
            data[randint(0, len(data) - 1)] = chr(getrandbits(8))
            data[randint(0, len(data) - 1)] = chr(getrandbits(8))
            data = ''.join(data)
            values = [code] + [self.get_value(a) for a in args]
            result = self.hypercall(values, 8, data)
            if result['exception']:
                self.stats_event('exception')
            else:
                self.stats_event(get_hypercall_status_extended(result['status']))
        self.stats_print('Hypercall status codes')
        return

    def fuzz_hypercalls_randomly(self, codes, iterations):
        for it in range(iterations):
            code = random.choice(codes)
            rule = self.hypercall_args.get(code, {})
            if not rule:
                self.msg("WARNING: Fuzzing rule is not defined for this hypercall!")
            args = rule.get('args', [])
            data = list('\x00' * 32)
            data[randint(0, len(data) - 1)] = chr(getrandbits(8))
            data[randint(0, len(data) - 1)] = chr(getrandbits(8))
            data = ''.join(data)
            values = [code] + [self.get_value(a) for a in args]
            result = self.hypercall(values, 8, data)
        return



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2017, Google
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#

import os
import struct
import tempfile
import unittest

from tests.software import util


class TestTpmEventLogChipsecUtil(util.TestChipsecUtil):
    """Test the tpm commands exposed by chipsec_utils."""

    def _tpm12_event(self, pcr_index, event_type, digest, event):
        _fmt = "II20sI"
        event_size = len(event)
        header = struct.pack(_fmt, pcr_index, event_type, digest, event_size)
        return header + event

    def _parse_eventlog(self, events):
        fileno, binary_event_log = tempfile.mkstemp()
        for event in events:
            os.write(fileno, event)
        os.close(fileno)
        self._chipsec_util("tpm parse_log {}".format(binary_event_log))
        os.remove(binary_event_log)

    def test_tpm_eventlog_basic(self):
        empty_event = self._tpm12_event(0x0, 0x0, b"\x00" * 20, b"")
        self._parse_eventlog([empty_event])
        self.assertIn(b"EV_PREBOOT_CERT", self.log)

    def test_tpm_eventlog_firmware_blob(self):
        data = struct.pack("QL", 0xABABABABFEFEFEFE, 0x12345678)
        blob_event = self._tpm12_event(0x0, 0x80000008, b"\x00" * 20, data)
        self._parse_eventlog([blob_event])
        self.assertIn(b"EV_EFI_PLATFORM_FIRMWARE_BLOB", self.log)
        self._assertLogValue("base", "0xababababfefefefe")
        self._assertLogValue("length", "0x12345678")


if __name__ == '__main__':
    unittest.main()



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2023, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#


# To execute: python -m unittest tests.helpers.test_replayhelper

import unittest
from os.path import join
import chipsec.helper.replay.replayhelper as rph


class ReplayHelperTest(unittest.TestCase):
    def setUp(self):
        unittest.TestCase.setUp(self)
        self.replayhelper = rph.ReplayHelper(join("tests", "helpers", "kblreplaytest.json"))
        self.assertTrue(self.replayhelper.create())
        self.assertTrue(self.replayhelper.start())

    def tearDown(self):
        unittest.TestCase.tearDown(self)
        self.assertTrue(self.replayhelper.stop())
        self.assertTrue(self.replayhelper.delete())

    def test_cpu_read_pci_reg_one_byte(self):
        pci_read_value = self.replayhelper.read_pci_reg(0, 0, 0, 0, 0x1)
        self.assertEqual(pci_read_value, 0x86)

    def test_cpu_read_pci_reg_two_bytes(self):
        pci_read_value = self.replayhelper.read_pci_reg(0, 0, 0, 0, 0x2)
        self.assertEqual(pci_read_value, 0x8086)

    def test_cpu_read_pci_reg_four_bytes(self):
        pci_read_value = self.replayhelper.read_pci_reg(0, 0, 0, 0, 0x4)
        self.assertEqual(pci_read_value, 0x59048086)

    def test_pch_read_pci_reg_one_byte(self):
        pci_read_value = self.replayhelper.read_pci_reg(0, 0x1f, 0, 0, 0x1)
        self.assertEqual(pci_read_value, 0x86)

    def test_pch_read_pci_reg_two_bytes(self):
        pci_read_value = self.replayhelper.read_pci_reg(0, 0x1f, 0, 0, 0x2)
        self.assertEqual(pci_read_value, 0x8086)

    def test_pch_read_pci_reg_four_bytes(self):
        pci_read_value = self.replayhelper.read_pci_reg(0, 0x1f, 0, 0, 0x4)
        self.assertEqual(pci_read_value, 0x9D4E8086)

    def test_get_thread_count(self):
        thread_count = self.replayhelper.get_threads_count()
        self.assertEqual(thread_count, 4)

    def test_read_phyis_mem_align(self):
        mem_value = self.replayhelper.read_phys_mem(0x5000, 0x2)
        self.assertEqual(mem_value, b'\xd1\x99')

    def test_read_phyis_mem_unaligned(self):
        mem_value = self.replayhelper.read_phys_mem(0x5001, 0x2)
        self.assertEqual(mem_value, b'\x99\xaa')

    def test_read_phyis_mem_oob(self):
        mem_value = self.replayhelper.read_phys_mem(0x5010, 0x2)
        self.assertEqual(mem_value, b'\xf3\x99')

    def test_cpuid_one(self):
        cpuid_value = self.replayhelper.cpuid(1, 0)
        self.assertEqual(cpuid_value, (526057, 51382272, 2147154879, 3219913727))

    def test_cpuid_two(self):
        cpuid_value = self.replayhelper.cpuid(2, 0)
        self.assertEqual(cpuid_value, (1979933441, 15775231, 0, 12779520))

    def test_write_pci_reg_new_location(self):
        self.replayhelper.write_pci_reg(0, 0x3, 0, 0x2D, 0x33, 0x1)
        pci_read_value = self.replayhelper.read_pci_reg(0, 0x3, 0, 0x2d, 0x1)
        self.assertEqual(pci_read_value, 0xFF)

    def test_write_pci_reg_defined_location(self):
        self.replayhelper.write_pci_reg(0, 0x0, 0, 0x0, 0x44, 0x1)
        pci_read_value = self.replayhelper.read_pci_reg(0, 0x0, 0, 0x0, 0x2)
        self.assertEqual(pci_read_value, 32902)

    def test_read_mmio_reg(self):
        mmio_read_value = self.replayhelper.read_mmio_reg(0xfed0, 0x3)
        self.assertEqual(mmio_read_value, 0xababab)

    def test_write_mmio_reg(self):
        self.replayhelper.write_mmio_reg(0x123, 0x1, 0x22)
        mmio_read_value = self.replayhelper.read_mmio_reg(0x123, 0x1)
        self.assertEqual(mmio_read_value, 0x22)



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2018, Eclypsium, Inc.
# Copyright (c) 2019-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

"""
This module checks that ME Manufacturing mode is not enabled.

References:

https://blog.ptsecurity.com/2018/10/intel-me-manufacturing-mode-macbook.html

`PCI_DEVS.H <https://github.com/coreboot/coreboot/blob/master/src/soc/intel/*/include/soc/pci_devs.h>`_

.. code-block::

    #define PCH_DEV_SLOT_CSE        0x16
    #define  PCH_DEVFN_CSE          _PCH_DEVFN(CSE, 0)
    #define  PCH_DEV_CSE            _PCH_DEV(CSE, 0)

https://github.com/coreboot/coreboot/blob/master/src/soc/intel/apollolake/cse.c

.. code-block::

    fwsts1 = dump_status(1, PCI_ME_HFSTS1);
    # Minimal decoding is done here in order to call out most important
    # pieces. Manufacturing mode needs to be locked down prior to shipping
    # the product so it's called out explicitly.
    printk(BIOS_DEBUG, "ME: Manufacturing Mode      : %s", (fwsts1 & (1 << 0x4)) ? "YES" : "NO");

`PCH.H <https://github.com/coreboot/coreboot/blob/master/src/southbridge/intel/*/pch.h>`_

.. code-block::

    #define PCH_ME_DEV                PCI_DEV(0, 0x16, 0)

`ME.H <https://github.com/coreboot/coreboot/blob/master/src/southbridge/intel/*/me.h>`_

.. code-block::

    struct me_hfs {
            u32 working_state: 4;
            u32 mfg_mode: 1;
            u32 fpt_bad: 1;
            u32 operation_state: 3;
            u32 fw_init_complete: 1;
            u32 ft_bup_ld_flr: 1;
            u32 update_in_progress: 1;
            u32 error_code: 4;
            u32 operation_mode: 4;
            u32 reserved: 4;
            u32 boot_options_present: 1;
            u32 ack_data: 3;
            u32 bios_msg_ack: 4;
    } __packed;

`ME_STATUS.C <https://github.com/coreboot/coreboot/blob/master/src/southbridge/intel/*/me_status.c>`_

.. code-block::

     printk(BIOS_DEBUG, "ME: Manufacturing Mode      : %s", hfs->mfg_mode ? "YES" : "NO");

This module checks the following:

    ``HFS.MFG_MODE BDF: 0:22:0 offset 0x40 - Bit [4]``

Usage:
    ``chipsec_main -m common.me_mfg_mode``

Examples:
    >>> chipsec_main.py -m common.me_mfg_mode

The module returns the following results:

    FAILED : HFS.MFG_MODE is set

    PASSED : HFS.MFG_MODE is not set.

Hardware registers used:
    - HFS.MFG_MODE
"""

from chipsec.module_common import BaseModule, HWCONFIG
from chipsec.library.returncode import ModuleResult
from typing import List

TAGS = [HWCONFIG]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'HWCONFIG']


class me_mfg_mode(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)

    def is_supported(self) -> bool:
        if self.cs.device.is_enabled('MEI1'):
            return True
        else:
            self.logger.log_important('MEI1 not enabled.  Skipping module.')
            return False

    def check_me_mfg_mode(self) -> int:
        me_mfg_mode_res = ModuleResult.FAILED
        me_hfs_reg = self.cs.register.read('HFS')
        me_mfg_mode = self.cs.register.get_field('HFS', me_hfs_reg, 'MFG_MODE')

        if 0 == me_mfg_mode:
            me_mfg_mode_res = ModuleResult.PASSED
            self.logger.log_passed('ME is not in Manufacturing Mode')
        else:
            self.logger.log_failed('ME is in Manufacturing Mode')
            self.result.setStatusBit(self.result.status.POTENTIALLY_VULNERABLE)

        return self.result.getReturnCode(me_mfg_mode_res)

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('ME Manufacturing Mode')
        self.res = self.check_me_mfg_mode()
        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Access to message bus (IOSF sideband) interface registers on Intel SoCs

References:

- Intel(R) Atom(TM) Processor D2000 and N2000 Series Datasheet, Volume 2, July 2012, Revision 003
  http://www.intel.com/content/dam/doc/datasheet/atom-d2000-n2000-vol-2-datasheet.pdf (section 1.10.2)

usage:
    >>> msgbus_reg_read( port, register )
    >>> msgbus_reg_write( port, register, data )
    >>> msgbus_read_message( port, register, opcode )
    >>> msgbus_write_message( port, register, opcode, data )
    >>> msgbus_send_message( port, register, opcode, data )
"""

from typing import Optional
from chipsec.hal import hal_base
from chipsec.library.exceptions import RegisterNotFoundError


#
# IOSF Message bus message opcodes
# Reference: http://lxr.free-electrons.com/source/arch/x86/include/asm/iosf_mbi.h
#
class MessageBusOpcode:
    MB_OPCODE_MMIO_READ = 0x00
    MB_OPCODE_MMIO_WRITE = 0x01
    MB_OPCODE_IO_READ = 0x02
    MB_OPCODE_IO_WRITE = 0x03
    MB_OPCODE_CFG_READ = 0x04
    MB_OPCODE_CFG_WRITE = 0x05
    MB_OPCODE_CR_READ = 0x06
    MB_OPCODE_CR_WRITE = 0x07
    MB_OPCODE_REG_READ = 0x10
    MB_OPCODE_REG_WRITE = 0x11
    MB_OPCODE_ESRAM_READ = 0x12
    MB_OPCODE_ESRAM_WRITE = 0x13

#
# IOSF Message bus unit ports
# Reference: http://lxr.free-electrons.com/source/arch/x86/include/asm/iosf_mbi.h
# @TODO: move these to per-platform XML config?
#


class MessageBusPort_Atom:
    UNIT_AUNIT = 0x00
    UNIT_SMC = 0x01
    UNIT_CPU = 0x02
    UNIT_BUNIT = 0x03
    UNIT_PMC = 0x04
    UNIT_GFX = 0x06
    UNIT_SMI = 0x0C
    UNIT_USB = 0x43
    UNIT_SATA = 0xA3
    UNIT_PCIE = 0xA6


class MessageBusPort_Quark:
    UNIT_HBA = 0x00
    UNIT_HB = 0x03
    UNIT_RMU = 0x04
    UNIT_MM = 0x05
    UNIT_SOC = 0x31


class MsgBus(hal_base.HALBase):

    def __init__(self, cs):
        super(MsgBus, self).__init__(cs)
        self.helper = cs.helper
        self.p2sbHide = None

    def __MB_MESSAGE_MCR(self, port: int, reg: int, opcode: int) -> int:
        mcr = 0x0
        mcr = self.cs.register.set_field('MSG_CTRL_REG', mcr, 'MESSAGE_WR_BYTE_ENABLES', 0xF)
        mcr = self.cs.register.set_field('MSG_CTRL_REG', mcr, 'MESSAGE_ADDRESS_OFFSET', reg)
        mcr = self.cs.register.set_field('MSG_CTRL_REG', mcr, 'MESSAGE_PORT', port)
        mcr = self.cs.register.set_field('MSG_CTRL_REG', mcr, 'MESSAGE_OPCODE', opcode)
        return mcr

    def __MB_MESSAGE_MCRX(self, reg: int) -> int:
        mcrx = 0x0
        mcrx = self.cs.register.set_field('MSG_CTRL_REG_EXT', mcrx, 'MESSAGE_ADDRESS_OFFSET_EXT', (reg >> 8), preserve_field_position=True)
        return mcrx

    def __MB_MESSAGE_MDR(self, data: int) -> int:
        mdr = 0x0
        mdr = self.cs.register.set_field('MSG_DATA_REG', mdr, 'MESSAGE_DATA', data)
        return mdr

    def __hide_p2sb(self, hide: bool) -> bool:
        if not self.p2sbHide:
            if self.cs.register.has_field("P2SBC", "HIDE"):
                self.p2sbHide = {'reg': 'P2SBC', 'field': 'HIDE'}
            elif self.cs.register.has_field("P2SB_HIDE", "HIDE"):
                self.p2sbHide = {'reg': 'P2SB_HIDE', 'field': 'HIDE'}
            else:
                raise RegisterNotFoundError('RegisterNotFound: P2SBC')

        hidden = not self.cs.device.is_enabled('P2SBC')
        if hide:
            self.cs.register.write_field(self.p2sbHide['reg'], self.p2sbHide['field'], 1)
        else:
            self.cs.register.write_field(self.p2sbHide['reg'], self.p2sbHide['field'], 0)
        return hidden

    #
    # Issues read message on the message bus
    #
    def msgbus_read_message(self, port: int, register: int, opcode: int) -> Optional[int]:
        mcr = self.__MB_MESSAGE_MCR(port, register, opcode)
        mcrx = self.__MB_MESSAGE_MCRX(register)

        self.logger.log_hal(f'[msgbus] Read: port 0x{port:02X} + 0x{register:08X} (op = 0x{opcode:02X})')
        self.logger.log_hal(f'[msgbus]       MCR = 0x{mcr:08X}, MCRX = 0x{mcrx:08X}')

        mdr_out = self.helper.msgbus_send_read_message(mcr, mcrx)

        self.logger.log_hal(f'[msgbus]       < 0x{mdr_out:08X}')

        return mdr_out

    #
    # Issues write message on the message bus
    #
    def msgbus_write_message(self, port: int, register: int, opcode: int, data: int) -> None:
        mcr = self.__MB_MESSAGE_MCR(port, register, opcode)
        mcrx = self.__MB_MESSAGE_MCRX(register)
        mdr = self.__MB_MESSAGE_MDR(data)

        self.logger.log_hal(f'[msgbus] Write: port 0x{port:02X} + 0x{register:08X} (op = 0x{opcode:02X}) < data = 0x{data:08X}')
        self.logger.log_hal(f'[msgbus]        MCR = 0x{mcr:08X}, MCRX = 0x{mcrx:08X}, MDR = 0x{mdr:08X}')

        return self.helper.msgbus_send_write_message(mcr, mcrx, mdr)

    #
    # Issues generic message on the message bus
    #
    def msgbus_send_message(self, port: int, register: int, opcode: int, data: Optional[int] = None) -> Optional[int]:
        mcr = self.__MB_MESSAGE_MCR(port, register, opcode)
        mcrx = self.__MB_MESSAGE_MCRX(register)
        mdr = None if data is None else self.__MB_MESSAGE_MDR(data)

        self.logger.log_hal(f'[msgbus] message: port 0x{port:02X} + 0x{register:08X} (op = 0x{opcode:02X})')
        if data is not None:
            self.logger.log_hal(f'[msgbus]          data = 0x{data:08X}')
        self.logger.log_hal(f'[msgbus]          MCR = 0x{mcr:08X}, MCRX = 0x{mcrx:08X}, MDR = 0x{mdr:08X}')

        mdr_out = self.helper.msgbus_send_message(mcr, mcrx, mdr)

        self.logger.log_hal(f'[msgbus]          < 0x{mdr_out:08X}')

        return mdr_out

    #
    # Message bus register read/write
    #

    def msgbus_reg_read(self, port: int, register: int) -> Optional[int]:
        return self.msgbus_read_message(port, register, MessageBusOpcode.MB_OPCODE_REG_READ)

    def msgbus_reg_write(self, port: int, register: int, data: int) -> None:
        return self.msgbus_write_message(port, register, MessageBusOpcode.MB_OPCODE_REG_WRITE, data)

    def mm_msgbus_reg_read(self, port: int, register: int) -> int:
        was_hidden = False
        if self.cs.register.is_defined('P2SBC'):
            was_hidden = self.__hide_p2sb(False)
        mmio_addr = self.cs.mmio.get_MMIO_BAR_base_address('SBREGBAR')[0]
        reg_val = self.cs.mmio.read_MMIO_reg_dword(mmio_addr, ((port & 0xFF) << 16) | (register & 0xFFFF))
        if self.cs.register.is_defined('P2SBC') and was_hidden:
            self.__hide_p2sb(True)
        return reg_val

    def mm_msgbus_reg_write(self, port: int, register: int, data: int) -> Optional[int]:
        was_hidden = False
        if self.cs.register.is_defined('P2SBC'):
            was_hidden = self.__hide_p2sb(False)
        mmio_addr = self.cs.mmio.get_MMIO_BAR_base_address('SBREGBAR')[0]
        reg_val = self.cs.mmio.write_MMIO_reg_dword(mmio_addr, ((port & 0xFF) << 16) | (register & 0xFFFF), data)
        if self.cs.register.is_defined('P2SBC') and was_hidden:
            self.__hide_p2sb(True)
        return reg_val



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
The mmcfg command allows direct access to memory mapped config space.

>>> chipsec_util mmcfg base
>>> chipsec_util mmcfg read <bus> <device> <function> <offset> <width>
>>> chipsec_util mmcfg write <bus> <device> <function> <offset> <width> <value>
>>> chipsec_util mmcfg ec


Examples:

>>> chipsec_util mmcfg base
>>> chipsec_util mmcfg read 0 0 0 0x200 4
>>> chipsec_util mmcfg write 0 0 0 0x200 1 0x1A
>>> chipsec_util mmcfg ec
"""

from chipsec.command import BaseCommand, toLoad
from argparse import ArgumentParser


# Access to Memory Mapped PCIe Configuration Space (MMCFG)
class MMCfgCommand(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.All

    def parse_arguments(self) -> None:
        parser = ArgumentParser(prog='chipsec_util mmcfg', usage=__doc__)
        subparsers = parser.add_subparsers()

        parser_base = subparsers.add_parser('base')
        parser_base.set_defaults(func=self.base)

        parser_read = subparsers.add_parser('read')
        parser_read.set_defaults(func=self.read)
        parser_read.add_argument('bus', type=lambda x: int(x, 16), help='Bus (hex)')
        parser_read.add_argument('device', type=lambda x: int(x, 16), help='Device (hex)')
        parser_read.add_argument('function', type=lambda x: int(x, 16), help='Function (hex)')
        parser_read.add_argument('offset', type=lambda x: int(x, 16), help='Offset (hex)')
        parser_read.add_argument('width', type=int, choices=[1, 2, 4], help='Width')

        parser_write = subparsers.add_parser('write')
        parser_write.set_defaults(func=self.write)
        parser_write.add_argument('bus', type=lambda x: int(x, 16), help='Bus (hex)')
        parser_write.add_argument('device', type=lambda x: int(x, 16), help='Device (hex)')
        parser_write.add_argument('function', type=lambda x: int(x, 16), help='Function (hex)')
        parser_write.add_argument('offset', type=lambda x: int(x, 16), help='Offset (hex)')
        parser_write.add_argument('width', type=int, choices=[1, 2, 4], help='Width')
        parser_write.add_argument('value', type=lambda x: int(x, 16), help='Value to write (hex)')

        # Print the pcie extended capabilities
        parser_ec = subparsers.add_parser('ec')
        parser_ec.set_defaults(func=self.ec)

        parser.parse_args(self.argv, namespace=self)

    def base(self):
        pciexbar, pciexbar_sz = self.cs.mmio.get_MMCFG_base_address()
        self.logger.log(f'[CHIPSEC] Memory Mapped Config Base: 0x{pciexbar:016X}')
        self.logger.log(f'[CHIPSEC] Memory Mapped Config Size: 0x{pciexbar_sz:016X}')

    def read(self):
        data = self.cs.mmio.read_mmcfg_reg(self.bus, self.device, self.function, self.offset, self.width)
        self.logger.log(f'[CHIPSEC] Reading MMCFG register ({self.bus:02d}:{self.device:02d}.{self.function:d} + 0x{self.offset:02X}): 0x{data:X}')

    def write(self):
        self.cs.mmio.write_mmcfg_reg(self.bus, self.device, self.function, self.offset, self.width, self.value)
        self.logger.log(f'[CHIPSEC] Writing MMCFG register ({self.bus:02d}:{self.device:02d}.{self.function:d} + 0x{self.offset:02X}): 0x{self.value:X}')

    def ec(self):
        devs = self.cs.pci.enumerate_devices()
        for (b, d, f, _, _, _) in devs:
            capabilities = self.cs.mmio.get_extended_capabilities(b, d, f)
            if capabilities:
                self.logger.log(f'Extended Capabilities for {b:02X}:{d:02X}.{f:X}:')
                for cap in capabilities:
                    self.logger.log(f'{cap}')
                    if cap.id == 0xB:
                        vsec = self.cs.mmio.get_vsec(b, d, f, cap.off)
                        self.logger.log(f'\t{vsec}')


commands = {'mmcfg': MMCfgCommand}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2024, AMD Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@amd.com
#


"""
This module verifies support for Rom Armor and SPI ROM protections.

Reference:


usage:
    ``chipsec_main -m common.rom_armor``

Examples:
    >>> chipsec_main.py -m common.rom_armor

"""

from chipsec.module_common import BaseModule, BIOS, SMM
from chipsec.library.returncode import ModuleResult
from typing import List

TAGS = [BIOS, SMM]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'ROM_ARMOR']

SMU_PSP_SMN_BASE = 0x3800000
SMU_PSP_MBOX_CMD_STATUS = 0x00010970


class rom_armor(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)

    def is_supported(self) -> bool:
        return self.cs.is_amd()

    def check_ROMAMOR(self) -> bool:
        reg_value = self.cs.psp.smu_read32(SMU_PSP_SMN_BASE + SMU_PSP_MBOX_CMD_STATUS)
        self.logger.log_information(f"PSP Mailbox Status 0x{reg_value:X}")
        reg_value = self.cs.psp.smu_read32(SMU_PSP_SMN_BASE + 0x109fc)
        self.logger.log_information(f"PSP Mailbox Features 0x{reg_value:X}")
        hsti = self.cs.psp.query_HSTI()
        self.logger.log_information(f"HSTI 0x{hsti:X}")
        return bool(hsti>>11)

    def check_RA_Fencing(self) -> int:
        # Confirm SPI Control Bass address is blocked
        spi_ctrl_bar = self.cs.pci.read_dword(0, 0x14, 3, 0xa0)

        if (spi_ctrl_bar == 0xFFFFFFFF):
            self.logger.log_good("SPI BAR access from host is blocked")
        else:
            self.logger.log_bad("Host is able to access SPI Bar")

        return ModuleResult.PASSED

    # --------------------------------------------------------------------------
    # run( module_argv )
    # Required function: run here all tests from this module
    # --------------------------------------------------------------------------
    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test("Rom Armor Verification")
        rom_armor_enabled = self.check_ROMAMOR()
        self.res = ModuleResult.FAILED
        if (not (rom_armor_enabled)):
            self.logger.log_failed("Rom Armor is not enabled.")
            return self.res
        else:
            self.logger.log_good("Rom Armor enabled.")
        self.res = self.check_RA_Fencing()

        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2016, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


# \defgroup config Platform Configuration
# chipsec/cfg/\<platform\>.py - configuration for a specific \<platform\>



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Hyper-V VMBus functionality
"""
import time
from struct import *
from random import *
from chipsec.modules.tools.vmm.common import *
from chipsec.modules.tools.vmm.hv.define import *
from chipsec.library.logger import *
from chipsec.library.file import *
from chipsec.module_common import *
from chipsec.hal.vmm import VMM
from chipsec.library.defines import *


class RingBuffer(BaseModuleDebug):
    def __init__(self):
        BaseModuleDebug.__init__(self)
        self.promt = 'RING BUFFER'
        self.signal = True
        self.send_size = 0
        self.base_addr = []
        self.pfn = []

    def __del__(self):
        BaseModuleDebug.__del__(self)
        self.dbg('Free kernel memory (pfn pages)')
        self.base_addr = []
        self.send_size = 0
        self.pfn = []

    ##
    # ringbuffer_alloc - allocates kernel memory for ring buffer
    ##
    def ringbuffer_alloc(self, pages=4):
        (va, pa) = self.cs.mem.alloc_physical_mem(pages << 12, 0xFFFFFFFFFFFFFFFF)
        self.base_addr.append(va)
        if pa != 0:
            for i in range(pages):
                self.pfn.append(pa + (i << 12))
            self.ringbuffer_init()
        self.send_size = pages >> 1
        return pa != 0

    ##
    # ringbuffer_init - init data and control structures in the ring buffer
    ##
    def ringbuffer_init(self):
        init_page = '\x00' * 0x1000
        for addr in self.pfn:
            self.cs.mem.write_physical_mem(addr, len(init_page), init_page)
        return

    ##
    # ringbuffer_copyfrom - routine to copy to source from ring buffer
    ##
    def ringbuffer_copyfrom(self, index, total):
        ring_data_page = self.send_size + 1
        ring_data_size = len(self.pfn) - self.send_size - 1
        data = ''
        while total > 0:
            page = ring_data_page + (index >> 12) % ring_data_size
            addr = self.pfn[page] + (index & 0xFFF)
            size = min(total, 0x1000 - (index & 0xFFF))
            data += self.cs.mem.read_physical_mem(addr, size)
            total -= size
            index += size
        return data

    ##
    # ringbuffer_copyto - routine to copy from source to ring buffer
    ##
    def ringbuffer_copyto(self, index, data):
        ring_data_page = 1
        ring_data_size = self.send_size - 1
        while data:
            page = ring_data_page + (index >> 12) % ring_data_size
            addr = self.pfn[page] + (index & 0xFFF)
            size = min(len(data), 0x1000 - (index & 0xFFF))
            self.cs.mem.write_physical_mem(addr, size, data[:size])
            data = data[size:]
            index += size
        return

    ##
    # ringbuffer_read - Read and advance the read index
    ##
    def ringbuffer_read(self):
        ring_data_size = (len(self.pfn) - self.send_size - 1) << 12
        buffer = self.cs.mem.read_physical_mem(self.pfn[self.send_size], 0x10)
        write_index, read_index, _, _ = unpack('<4L', buffer)
        delta = write_index - read_index
        avail = delta if delta >= 0 else ring_data_size + delta
        if avail == 0:
            return ''
        header = self.ringbuffer_copyfrom(read_index, 16)
        pksize = 8 * unpack('<4HQ', header)[2] + 8
        buffer = self.ringbuffer_copyfrom(read_index, pksize)
        read_index = (read_index + pksize) % ring_data_size
        self.cs.mem.write_physical_mem(self.pfn[self.send_size] + 4, 4, DD(read_index))
        return buffer[:pksize - 8]

    ##
    # ringbuffer_write - Write to the ring buffer
    ##
    def ringbuffer_write(self, data):
        ring_data_size = (self.send_size - 1) << 12
        buffer = self.cs.mem.read_physical_mem(self.pfn[0], 0x10)
        write_index, read_index, _, _ = unpack('<4L', buffer)
        delta = read_index - write_index
        avail = delta if delta > 0 else ring_data_size + delta
        data += DD(0) + DD(write_index)
        if avail < len(data):
            return False
        self.ringbuffer_copyto(write_index, data)
        write_index = (write_index + len(data)) % ring_data_size
        self.cs.mem.write_physical_mem(self.pfn[0] + 0, 4, DD(write_index))
        self.signal = True
        return True

    def ringbuffer_read_with_timeout(self, timeout=0):
        self.dbg('Read and advance the read index ...')
        start_time = time.time()
        polling = True
        while polling:
            message = self.ringbuffer_read()
            polling = ((timeout == 0) or (time.time() - start_time <= timeout)) and not message
        return message

    def ringbuffer_write_with_timeout(self, message, timeout=0):
        self.dbg('Write to the ring buffer ...')
        start_time = time.time()
        polling = True
        while polling:
            result = self.ringbuffer_write(message)
            polling = ((timeout == 0) or (time.time() - start_time <= timeout)) and not result
        return result


class HyperV(BaseModuleDebug):
    def __init__(self):
        BaseModuleDebug.__init__(self)
        self.hypercall = VMM(self.cs)
        self.hypercall.init()
        self.membuf = self.cs.mem.alloc_physical_mem(4 * 0x1000, 0xFFFFFFFF)
        self.cs.mem.write_physical_mem(self.membuf[1], 4 * 0x1000, b'\x00' * 4 * 0x1000)
        self.old_sint2 = []
        self.old_simp = []
        self.old_siefp = []
        self.simp = []
        self.siefp = []

    def __del__(self):
        BaseModuleDebug.__del__(self)
        self.dbg('Free kernel memory')
        # if self.membuf[0] != 0:
        #    self.cs.mem.free_physical_mem(self.membuf[0])
        if len(self.old_sint2) == 2:
            self.cs.msr.write_msr(0, HV_X64_MSR_SINT2, self.old_sint2[0], self.old_sint2[1])
        if len(self.old_simp) == 2:
            self.cs.msr.write_msr(0, HV_X64_MSR_SIMP, self.old_simp[0], self.old_simp[1])
        if len(self.old_siefp) == 2:
            self.cs.msr.write_msr(0, HV_X64_MSR_SIEFP, self.old_siefp[0], self.old_siefp[1])
        for i in [x for x in self.ringbuffers]:
            del self.ringbuffers[i]

    ##
    # hv_init
    ##
    def hv_init(self):
        self.old_sint2 = self.cs.msr.read_msr(0, HV_X64_MSR_SINT2)
        self.old_simp = self.cs.msr.read_msr(0, HV_X64_MSR_SIMP)
        self.old_siefp = self.cs.msr.read_msr(0, HV_X64_MSR_SIEFP)
        pa = self.membuf[1]
        self.sint3 = self.cs.msr.read_msr(0, HV_X64_MSR_SINT3)
        self.cs.msr.write_msr(0, HV_X64_MSR_SINT2, self.sint3[0], self.sint3[1])
        self.cs.msr.write_msr(0, HV_X64_MSR_SIEFP, (pa & 0xFFFFFFFF) | 0x1, pa >> 32)
        # self.cs.msr.write_msr(0, HV_X64_MSR_SCONTROL, 0x1, 0x0)
        self.simp = self.cs.msr.read_msr(0, HV_X64_MSR_SIMP)
        self.siefp = self.cs.msr.read_msr(0, HV_X64_MSR_SIEFP)
        self.simp = (self.simp[0] + (self.simp[1] << 32)) & 0xFFFFFFFFFFFFF000
        self.siefp = (self.siefp[0] + (self.siefp[1] << 32)) & 0xFFFFFFFFFFFFF000
        return

    ##
    # hv_post_msg - Send a message
    ##
    def hv_post_msg(self, message):
        retries = 3
        hciv = hv_hciv(0, 0, HV_POST_MESSAGE)
        while retries > 0:
            result = self.hypercall.hypercall64_memory_based(hciv, message[:0x100]) & 0xFFFF
            if result == HV_STATUS_INSUFFICIENT_BUFFERS:
                retries -= 1
                time.sleep(0.1)
            else:
                retries = 0
        return result

    ##
    # hv_recv_msg - recieve message if exist otherwise empty string
    ##
    def hv_recv_msg(self, sint):
        buffer = self.cs.mem.read_physical_mem(self.simp + 0x100 * sint, 0x100)
        message_type, payload_size, message_flags = unpack('<LBB', buffer[0:6])
        if message_type == HVMSG_NONE:
            buffer = ''
        else:
            self.cs.mem.write_physical_mem(self.simp + 0x100 * sint, 0x4, DD(HVMSG_NONE))
            if message_flags & 0x1:
                self.cs.msr.write_msr(0, HV_X64_MSR_EOM, 0x0, 0x0)
        return buffer

    ##
    # hv_signal_event - send an event notification
    ##
    def hv_signal_event(self, connection_id, flag_number):
        hciv = hv_hciv(0, 0, HV_SIGNAL_EVENT)
        buffer = pack('<LHH', connection_id, flag_number, 0x0)
        result = self.hypercall.hypercall64_memory_based(hciv, buffer) & 0xFFFF
        return result

    ##
    # hv_recv_events - recieve all current events
    ##
    def hv_recv_events(self, sint):
        events = set()
        buffer = self.cs.mem.read_physical_mem(self.siefp + 0x100 * sint, 0x100)
        buffer = unpack('<64L', buffer)
        for i in range(64):
            if buffer[i]:
                for n in range(32):
                    if (buffer[i] >> n) & 0x1:
                        events.add(i * 32 + n)
        return events


class VMBus(HyperV):
    def __init__(self):
        HyperV.__init__(self)
        self.promt = 'VMBUS'
        self.onmessage_timeout = 0.1  # 0.02
        self.int_page = self.membuf[1] + 1 * 0x1000
        self.monitor_page1 = self.membuf[1] + 2 * 0x1000
        self.monitor_page2 = self.membuf[1] + 3 * 0x1000
        self.recv_int_page = self.int_page + 0x000
        self.send_int_page = self.int_page + 0x800
        self.supported_versions = {}
        self.offer_channels = {}
        self.open_channels = {}
        self.created_gpadl = {}
        self.ringbuffers = {}
        self.next_gpadl = 0x200E1E10

    ##
    # vmbus_init
    ##
    def vmbus_init(self):
        self.hv_init()
        self.vmbus_clear()
        return

    ##
    # vmbus_clear
    ##
    def vmbus_clear(self):
        # Purge hypervisor message queue
        while self.vmbus_recv_msg(0.1):
            pass
        return

    ##
    # vmbus_get_next_gpadl
    ##
    def vmbus_get_next_gpadl(self):
        self.next_gpadl += 1
        return self.next_gpadl

    ##
    # vmbus_post_msg - send a msg on the vmbus's message connection
    ##
    def vmbus_post_msg(self, message):
        if len(message) > 240:
            self.err(f'vmbus_post_msg: message it too long {len(message):d} bytes')
            message = message[:240]
        header = pack('<4L', VMBUS_MESSAGE_CONNECTION_ID, 0x0, 0x1, len(message))
        result = self.hv_post_msg(header + message)
        if result != HV_STATUS_SUCCESS:
            status = hypercall_status_codes[result] if result in hypercall_status_codes else ''
            self.err(f'vmbus_post_msg returns  {result:02X} {status}')
        return result == HV_STATUS_SUCCESS

    ##
    # vmbus_recv_msg - recieve message. it may return empty string in case of timeout
    ##
    def vmbus_recv_msg(self, timeout=0):
        start_time = time.time()
        polling = True
        while polling:
            message = self.hv_recv_msg(VMBUS_MESSAGE_SINT)
            polling = ((timeout == 0) or (time.time() - start_time <= timeout)) and not message
        if message:
            msg_type, payload_size, msg_flags, rsvd, port_id = unpack('<LBBHQ', message[0:16])
            if msg_type not in [HVMSG_NONE, 0x0001]:
                status = hv_message_type[msg_type] if msg_type in hv_message_type else ''
                self.fatal(f'vmbus_recv_msg: unhandeled message type 0x{msg_type:08X} {status}')
            if (payload_size < 8) or (payload_size > 240):
                self.msg(f'vmbus_recv_msg: invalid payload size {payload_size:d}')
                payload_size = 240
            if rsvd != 0x0000:
                self.msg(f'vmbus_recv_msg: invalid reserved field 0x{rsvd:04X}')
            # if port_id != VMBUS_MESSAGE_PORT_ID:
            #    self.msg('vmbus_recv_msg: invalid ConnectionID 0x%016x' % port_id)
            message = message[16: 16 + payload_size]
        return message

    ##
    # vmbus_setevent - Trigger an event notification on the specified channel
    ##
    def vmbus_setevent(self, child_relid):
        self.dbg('Trigger an event notification on the specified channel ...')
        if child_relid not in self.open_channels:
            self.err(f'vmbus_setevent could not find channel with child relid: {child_relid:d}')
            return False
        channel = self.open_channels[child_relid]['offer']
        if channel['monitor_allocated'] == 1:
            monitor_bit = channel['monitor_id'] & 0x1F
            monitor_grp = channel['monitor_id'] >> 5
            trigger_group_offset = 8 + 8 * monitor_grp
            self.cs.mem.set_mem_bit(self.send_int_page, child_relid)
            self.cs.mem.set_mem_bit(self.monitor_page2 + trigger_group_offset, monitor_bit)
        else:
            # Send an event notification to the parent
            self.dbg('Send an event notification to the parent ...')
            self.cs.mem.set_mem_bit(self.send_int_page, child_relid)
            result = self.hv_signal_event(channel['connection_id'], 0x0)
            if result != 0:
                status = hypercall_status_codes[result] if result in hypercall_status_codes else ''
                self.err(f'vmbus_setevent returns  {result:02X} {status}')
                return False
        return True

    ##
    # vmbus_recv_events - recieve all current events
    ##
    def vmbus_recv_events(self):
        return self.hv_recv_events(VMBUS_MESSAGE_SINT)

    ##
    # vmbus_get_next_version - returns the next version
    ##
    def vmbus_get_next_version(self, current_version):
        versions = {VERSION_WIN8_1: VERSION_WIN8, VERSION_WIN8: VERSION_WIN7, VERSION_WS2008: VERSION_INVAL}
        return versions[current_version] if current_version in versions else VERSION_INVAL

    ##
    # vmbus_connect - Sends a connect request on the partition service connection
    ##
    def vmbus_connect(self, vmbus_version=VERSION_WIN8, target_vcpu=0x0):
        self.dbg('Sending channel initiate msg ...')
        channel_message_header = pack('<LL', CHANNELMSG_INITIATE_CONTACT, 0x0)
        channel_initiate_contact = pack('<LLQQQ', vmbus_version, target_vcpu, self.int_page, self.monitor_page1, self.monitor_page2)
        result = self.vmbus_post_msg(channel_message_header + channel_initiate_contact)
        if result:
            result = self.vmbus_onmessage() == CHANNELMSG_VERSION_RESPONSE
        return result

    ##
    # vmbus_establish_gpadl - Estabish a GPADL for the specified buffer
    ##
    def vmbus_establish_gpadl(self, child_relid, gpadl, pfn):
        self.dbg('Estabish a GPADL for the specified buffer ...')
        byte_offset = 0
        byte_count = len(pfn) << 12
        pfn_array = ''.join([DQ(addr >> 12) for addr in pfn])
        gpa_range = pack('<LL', byte_count, byte_offset) + pfn_array
        rangecount = 0x1
        range_buflen = len(gpa_range)
        channel_message_header = pack('<LL', CHANNELMSG_GPADL_HEADER, 0x0)
        channel_gpadl_header = pack('<LLHH', child_relid, gpadl, range_buflen, rangecount)
        result = self.vmbus_post_msg(channel_message_header + channel_gpadl_header + gpa_range[:27 * 8])
        gpa_range = gpa_range[27 * 8:]
        while result and gpa_range != '':
            channel_message_header = pack('<LL', CHANNELMSG_GPADL_BODY, 0x0)
            channel_gpadl_body = pack('<LL', 0x0, gpadl)
            result = self.vmbus_post_msg(channel_message_header + channel_gpadl_body + gpa_range[:28 * 8])
            gpa_range = gpa_range[28 * 8:]
        if result:
            result = self.vmbus_onmessage() == CHANNELMSG_GPADL_CREATED
        return result

    ##
    # vmbus_teardown_gpadl - Teardown the specified GPADL handle
    ##
    def vmbus_teardown_gpadl(self, child_relid, gpadl):
        self.dbg('Teardown the specified GPADL handle ...')
        channel_message_header = pack('<LL', CHANNELMSG_GPADL_TEARDOWN, 0x0)
        channel_gpadl_teardown = pack('<LL', child_relid, gpadl)
        result = self.vmbus_post_msg(channel_message_header + channel_gpadl_teardown)
        if result:
            msgtype = self.vmbus_onmessage()
            if msgtype == CHANNELMSG_RESCIND_CHANNELOFFER:
                self.vmbus_process_rescind_offer(child_relid)
                result = result and (self.vmbus_onmessage() == CHANNELMSG_GPADL_TORNDOWN)
                result = result and (self.vmbus_onmessage() == CHANNELMSG_OFFERCHANNEL)
            else:
                result = (msgtype == CHANNELMSG_GPADL_TORNDOWN)

        return result

    ##
    # vmbus_open - Open the specified channel
    ##
    def vmbus_open(self, child_relid, gpadl, pageoffset=2, userdata='\x00' * 120):
        self.dbg('Open the specified channel ...')
        openid = child_relid
        target_vp = 0x0
        channel_message_header = pack('<LL', CHANNELMSG_OPENCHANNEL, 0x0)
        channel_open_channel = pack('<5L', child_relid, openid, gpadl, target_vp, pageoffset)
        result = self.vmbus_post_msg(channel_message_header + channel_open_channel + userdata)
        if result:
            result = self.vmbus_onmessage() == CHANNELMSG_OPENCHANNEL_RESULT
        return result

    ##
    # vmbus_close - Close the specified channel
    ##
    def vmbus_close(self, child_relid):
        self.dbg('Close the specified channel ...')
        channel_message_header = pack('<LL', CHANNELMSG_CLOSECHANNEL, 0x0)
        channel_close_channel = pack('<L', child_relid)
        return self.vmbus_post_msg(channel_message_header + channel_close_channel)

    ##
    # vmbus_disconnect - Sends a disconnect request on the partition service connection
    ##
    def vmbus_disconnect(self):
        self.dbg('Sending a disconnect request ...')
        channel_message_header = pack('<LL', CHANNELMSG_UNLOAD, 0x0)
        result = self.vmbus_post_msg(channel_message_header)
        result = result and (self.vmbus_onmessage() == 17)
        return result

    ##
    # vmbus_request_offers - Send a request to get all our pending offers
    ##
    def vmbus_request_offers(self):
        self.dbg('Sending a request to get all our pending offers ...')
        channel_message_header = pack('<LL', CHANNELMSG_REQUESTOFFERS, 0x0)
        result = self.vmbus_post_msg(channel_message_header)
        while result:
            msgtype = self.vmbus_onmessage()
            if msgtype != CHANNELMSG_OFFERCHANNEL:
                break
        return result and (msgtype == CHANNELMSG_ALLOFFERS_DELIVERED)

    ##
    # vmbus_process_rescind_offer - Rescind the offer by initiating a device removal
    ##
    def vmbus_process_rescind_offer(self, child_relid):
        self.dbg('Rescind the offer by initiating a device removal ...')
        channel_message_header = pack('<LL', CHANNELMSG_RELID_RELEASED, 0x0)
        channel_relid_released = pack('<L', child_relid)
        return self.vmbus_post_msg(channel_message_header + channel_relid_released)

    ##
    # vmbus_onmessage - Handler for channel protocol messages.
    ##
    def vmbus_onmessage(self):
        msgtype = CHANNELMSG_INVALID

        channelmsg = {
            CHANNELMSG_OFFERCHANNEL: 188,
            CHANNELMSG_ALLOFFERS_DELIVERED: 0,
            CHANNELMSG_GPADL_CREATED: 12,
            CHANNELMSG_OPENCHANNEL_RESULT: 12,
            CHANNELMSG_GPADL_TORNDOWN: 4,
            CHANNELMSG_RESCIND_CHANNELOFFER: 4,
            CHANNELMSG_VERSION_RESPONSE: 8
        }

        message = self.vmbus_recv_msg(self.onmessage_timeout)

        if len(message) == 0:
            self.msg('vmbus_onmessage: timeout')
        elif len(message) >= 8:
            msgtype, padding = unpack('<LL', message[:8])
            message_body = message[8:]

            self.dbg(f'vmbus_onmessage: message {msgtype:d} {vmbus_channel_message_type[msgtype]}')

            if msgtype not in channelmsg:
                self.msg(f'vmbus_onmessage: invalid message type {msgtype:d}')
                self.hex('Message', message)
            elif channelmsg[msgtype] > len(message_body):
                self.msg('vmbus_onmessage: message is too short!')
                self.hex('Message', message)
                exit(1)

            if padding != 0x00000000:
                self.msg(f'vmbus_onmessage invalid padding {padding:d}')

        # vmbus_ongpadl_created - GPADL created handler
        if msgtype == CHANNELMSG_GPADL_CREATED:
            child_relid, gpadl, status = unpack('<3L', message_body[:12])
            self.created_gpadl[gpadl] = {'child_relid': child_relid, 'status': status}
        # vmbus_onopen_result - Open result handler
        elif msgtype == CHANNELMSG_OPENCHANNEL_RESULT:
            child_relid, openid, status = unpack('<3L', message_body[:12])
            self.open_channels[child_relid] = {'openid': openid, 'status': status}
            offer = 'none'
            for i in self.offer_channels.keys():
                if self.offer_channels[i]['child_relid'] == child_relid:
                    offer = self.offer_channels[i]
                    break
            self.open_channels[child_relid]['offer'] = offer
        # vmbus_onversion_response - Version response handler
        elif msgtype == CHANNELMSG_VERSION_RESPONSE:
            version_supported, version = unpack('<2L', message_body[:8])
            if version_supported != 0x00:
                self.supported_versions[version] = 0x1
        # vmbus_onoffer - Handler for channel offers from vmbus in parent partition.
        elif msgtype == CHANNELMSG_OFFERCHANNEL:
            offer = message_body
            channel = {}
            uuid1 = uuid(offer[0x00:0x10])
            uuid2 = uuid(offer[0x10:0x20])
            # struct vmbus_channel_offer
            guid1 = hv_guid_desc[uuid1] if uuid1 in hv_guid_desc else 'Unknown'
            channel['name'] = guid1
            channel['flags'] = unpack('<H', offer[0x30: 0x32])[0]
            channel['mmio'] = unpack('<H', offer[0x32: 0x34])[0]
            channel['userdef'] = offer[0x34: 0xAC]
            channel['sub_channel'] = unpack('<H', offer[0xAC: 0xAE])[0]
            # struct vmbus_channel_offer_channel (continue)
            channel['child_relid'] = unpack('<L', offer[0xB0: 0xB4])[0]
            channel['monitor_id'] = unpack('<B', offer[0xB4: 0xB5])[0]
            channel['monitor_allocated'] = unpack('<B', offer[0xB5: 0xB6])[0] & 0x1
            channel['dedicated_interrupt'] = unpack('<H', offer[0xB6: 0xB8])[0] & 0x1
            channel['connection_id'] = unpack('<L', offer[0xB8: 0xBC])[0]
            self.offer_channels[offer[0x00:0x20]] = channel

        return msgtype

    ##
    # vmbus_recvpacket - Retrieve the user packet on the specified channel
    ##
    def vmbus_recvpacket(self, child_relid):
        self.dbg('Retrieve the user packet on the specified channel ...')
        rb = self.ringbuffers[child_relid]
        buffer = rb.ringbuffer_read_with_timeout(self.onmessage_timeout)
        if len(buffer) >= 16:
            packet_type, offset8, len8, flags, requestid = unpack('<4HQ', buffer[:16])
            buffer = buffer[16:]
        return buffer

    ##
    # vmbus_sendpacket - Send the specified buffer on the given channel
    ##
    def vmbus_sendpacket(self, child_relid, data, requestid, packet_type, flags):
        self.dbg('Send the specified buffer on the given channel ...')
        rb = self.ringbuffers[child_relid]
        while (len(data) & 0x7) != 0:
            data += '\x00'
        offset8 = 16 >> 3
        len8 = offset8 + (len(data) >> 3)
        vmpacket_descriptor = pack('<4HQ', packet_type, offset8, len8, flags, requestid)
        rb.ringbuffer_write(vmpacket_descriptor + data)
        if rb.signal:
            self.vmbus_setevent(child_relid)
        return

    ##
    # vmbus_sendpacket_pagebuffer - Send a range of single-page buffer
    ##
    def vmbus_sendpacket_pagebuffer(self):
        self.dbg('Send a range of single-page buffer ...')
        return

    ##
    # vmbus_sendpacket_multipagebuffer - Send a multi-page buffer packet
    ##
    def vmbus_sendpacket_multipagebuffer(self):
        self.dbg('Send a multi-page buffer packet ...')
        return

    ##
    # vmbus_recvpacket_raw - Retrieve the raw packet on the specified channel
    ##
    def vmbus_recvpacket_raw(self):
        self.dbg('Retrieve the raw packet on the specified channel ...')
        return


class VMBusDiscovery(VMBus):
    def __init__(self):
        VMBus.__init__(self)

    def __del__(self):
        VMBus.__del__(self)

    ##
    # vmbus_rescind_all_offers - Rescind all offers by initiating a device removal
    ##
    def vmbus_rescind_all_offers(self):
        # for i in self.offer_channels.keys():
        #    relid = self.offer_channels[i]['child_relid']
        #    self.vmbus_process_rescind_offer(relid)
        for i in range(0x10):
            self.vmbus_process_rescind_offer(i)
        return

    ##
    # get_relid_by_guid
    ##
    def get_relid_by_guid(self, guid):
        relid = 0
        for i in self.offer_channels.keys():
            if (guid == uuid(i[0x00:0x10])) or (guid == uuid(i[0x10:0x20])):
                relid = self.offer_channels[i]['child_relid']
                break
        return relid

    ##
    # scan_supported_versions
    ##
    def scan_supported_versions(self, mask=0x000F000F):
        version = 0x00000000
        while True:
            self.vmbus_connect(version)
            if version == mask:
                break
            version = (~(~version & mask) + 1) & mask
        return

    ##
    # scan_physical_addresses
    ##
    def scan_physical_addresses(self, version):
        pages = (self.int_page, self.monitor_page1, self.monitor_page2)
        FFs = 0xFFFFFFFFFFFFFFFF
        for i in range(64):
            self.supported_versions = {}
            self.int_page = (FFs << (63 - i)) & FFs
            self.dbg(f'Address: 0x{self.int_page:016X}')
            self.vmbus_connect(version)
            self.logger.log(str(self.supported_versions))
        (self.int_page, self.monitor_page1, self.monitor_page2) = pages
        return

    ##
    # print_supported_versions
    ##
    def print_supported_versions(self):
        self.msg('')
        self.msg('******************** Supported versions ********************')
        for version in sorted(self.supported_versions.keys()):
            status = 'Unknown' if version not in vmbus_versions else vmbus_versions[version]
            self.msg(f'  {version >> 16:d} . {version & 0xFFFF:2d} - {status}')
        return

    ##
    # print_offer_channels - Print offered channels
    ##
    def print_offer_channels(self):
        self.msg('')
        self.msg('******************** Offered channels **********************')
        uuid_sorted_by_connid = dict((value['connection_id'], key) for (key, value) in self.offer_channels.items()).values()
        for i in uuid_sorted_by_connid:
            channel = self.offer_channels[i]
            flags = []
            for n in range(16):
                if (n in channel_flags) and (((channel['flags'] >> n) & 0x1) == 0x1):
                    flags.append(channel_flags[n])

            conid = f'Connection ID: 0x{channel["connection_id"]:08X}'
            relid = f'Child relid: 0x{channel["child_relid"]:08X}'
            mmios = f'MMIO: {channel["mmio"]:d}MB'
            subch = f'Sub channel: 0x{channel["sub_channel"]:04X}'
            monid = f'Monitor: {channel["monitor_allocated"]:d} ID=0x{channel["monitor_id"]:02X}'
            dintr = f'Dedicated interrupt: {channel["dedicated_interrupt"]:d}'
            flags = f'Flags: 0x{channel["flags"]:04X} >{", ".join(flags)}'

            self.msg('')
            self.msg(f'{channel["name"]}')
            self.msg(f'  Hardware IDs:  {uuid(i[0x00:0x10])}   {uuid(i[0x10:0x20])}')
            self.msg(f'  {conid}   {relid}   {subch}   {monid}')
            self.msg(f'  {mmios}   {dintr}   {flags}')
        return

    ##
    # print_created_gpadl
    ##
    def print_created_gpadl(self):
        self.msg('')
        self.msg('******************** Created GPADLs ************************')
        self.msg('  gpadl        |  child_relid  |  creation_status  ')
        self.msg('---------------------------------------------------')
        for gpadl in sorted(self.created_gpadl.keys()):
            channel = self.created_gpadl[gpadl]
            self.msg(f'  0x{gpadl:08X}   |  0x{channel["child_relid"]:08X}   |  0x{channel["status"]:08X}')
        self.msg('---------------------------------------------------')
        return

    ##
    # print_open_channels
    ##
    def print_open_channels(self):
        self.msg('')
        self.msg('******************** Open Channels *************************')
        self.msg('  child_relid  |  openid       |  status           ')
        self.msg('---------------------------------------------------')
        for child_relid in sorted(self.open_channels):
            channel = self.open_channels[child_relid]
            self.msg(f'  0x{child_relid:08X}   |  0x{channel["openid"]:08X}   |  0x{channel["status"]:08X}')
        self.msg('---------------------------------------------------')
        return

    ##
    # print_events
    ##
    def print_events(self):
        events = self.vmbus_recv_events()
        result = []
        for i in events:
            result.append(f'{i:02X}')
        if len(result) != 0:
            self.msg('EVENTS: ' + ', '.join(result))
        return



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2024, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

class SGX_Check_Sidekick():
    def __init__(self, cs):
        self.cs = cs
        self.sgx_global_en_defined = self.cs.register.has_field('IA32_FEATURE_CONTROL', 'SGX_GLOBAL_EN')

    def check_valid(self, tid) -> bool:
        if self.sgx_global_en_defined:
            sgx_global_en = self.cs.register.read_field('IA32_FEATURE_CONTROL', 'SGX_GLOBAL_EN', tid)
        else:
            sgx_global_en = True
        return sgx_global_en



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2020, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
Extended Page Table (EPT) Finder

Usage:
   ``chipsec_main -m tools.vmm.ept_finder [-a dump,<file_name>|file,<file_name>,<revision_id>]``

    - ``dump``          : Dump contents
    - ``file``          : Load contents from file
    - ``<file_name>``   : File name to read from or dump to
    - ``<revision_id>`` : Revision ID (hex)

Where:
    - ``[]``: optional line

Examples:
    >>> chipsec_main.py -i -m tools.vmm.ept_finder
    >>> chipsec_main.py -i -m tools.vmm.ept_finder -a dump,my_file.bin
    >>> chipsec_main.py -i -m tools.vmm.ept_finder -a file,my_file.bin,0x0

.. important::
    - This module is designed to run in a VM environment
    - Behavior on physical HW is undefined

"""

import os
import struct
import glob

from chipsec.library.logger import logger
from chipsec.library.file import write_file
from chipsec.module_common import BaseModule
from chipsec.library.returncode import ModuleResult
from chipsec.hal.paging import c_extended_page_tables


class c_extended_page_tables_from_file(c_extended_page_tables):
    def __init__(self, cs, read_from_file, par):
        c_extended_page_tables.__init__(self, cs)
        self.read_from_file = read_from_file
        self.par = par

    def readmem(self, name, addr, size=4096):
        if self.read_from_file:
            for (pa, end_pa, source) in self.par:
                if (pa <= addr) and (addr + size <= end_pa):
                    source.seek(addr - pa)
                    return source.read(size)
            logger().log_error(f'Invalid memory address: {addr:016x}-{addr + size:016x}')
            return '\xFF' * size
        return self.cs.mem.read_physical_mem(addr, size)


class ept_finder(BaseModule):
    def __init__(self):
        BaseModule.__init__(self)
        self.read_from_file = False
        self.par = []

    def read_physical_mem(self, addr, size=0x1000):
        if self.read_from_file:
            for (pa, end_pa, source) in self.par:
                if (pa <= addr) and (addr + size <= end_pa):
                    source.seek(addr - pa)
                    return source.read(size)
            self.logger.log_error(f'Invalid memory address: {addr:016x}-{addr + size:016x}')
            return '\xFF' * size
        return self.cs.mem.read_physical_mem(addr, size)

    def read_physical_mem_dword(self, addr):
        return struct.unpack('<L', self.read_physical_mem(addr, 4))[0]

    def get_memory_ranges(self):
        MASK = 0xFFFFFFFFFFFFF000
        tsegmb = None
        touud = None

        if self.cs.register.is_defined('PCI0.0.0_TSEGMB'):
            tsegmb = self.cs.register.read('PCI0.0.0_TSEGMB') & MASK
        else:
            self.logger.log_error('Couldn not find definition of required registers: TSEGMB')

        if self.cs.register.is_defined('PCI0.0.0_TOUUD'):
            touud = self.cs.register.read('PCI0.0.0_TOUUD') & MASK
        else:
            self.logger.log_error('Could not find definition of required registers: TOUUD')

        par = []
        if not (tsegmb is None):
            par.append((0x00000000, tsegmb, None))
        if not (touud is None):
            par.append((0x100000000, touud, None))

        return par

    def find_vmcs_by_ept(self, ept_list, revision_id):
        EPTP_OFFSET = 0x0140
        MASK = 0xFFFFFFFFFFFFF000
        vmcs_list = []
        for (pa, end_pa, _) in self.par:
            while pa < end_pa:
                revid = self.read_physical_mem_dword(pa)
                eptp = self.read_physical_mem_dword(pa + EPTP_OFFSET)
                eptp += self.read_physical_mem_dword(pa + EPTP_OFFSET + 4) << 32
                if (eptp & MASK in ept_list) and (revision_id == revid):
                    vmcs_list.append(pa)
                pa += 0x1000
        return vmcs_list

    def find_ept_pt(self, pt_addr_list, mincount, level):
        pt_list = {}
        for (pa, end_pa, _) in self.par:
            while pa < end_pa:
                page = struct.unpack('<512Q', self.read_physical_mem(pa))
                count = 0
                allzeros = True
                topalike = True
                reserved = False
                for i in range(512):
                    big_page = ((page[i] >> 7) & 0x1) == 1
                    memtype = ((page[i] >> 3) & 0x7)

                    if level == 4:
                        reserved_bits_mask = 0x000FFF0000000000
                    elif level == 3:
                        if big_page:
                            reserved_bits_mask = 0x000FFF00001FF000
                        else:
                            reserved_bits_mask = 0x000FFF0000000078
                    elif level == 2:
                        if big_page:
                            reserved_bits_mask = 0x000FFF003FFFF000
                        else:
                            reserved_bits_mask = 0x000FFF0000000078
                    elif level == 1:
                        reserved_bits_mask = 0x000FFF00000000F8

                    if (page[i] & reserved_bits_mask) != 0:
                        reserved = True
                        break

                    if (level == 4) or (level in [2, 3] and big_page):
                        if memtype not in [0, 1, 4, 5, 6]:
                            reserved = True
                            break

                    if page[i] != 0:
                        allzeros = False
                        if i >= 8:
                            topalike = False

                    if (page[i] & 0x0000FFFFFFFFF000) in pt_addr_list:
                        count += 1

                if not reserved and not allzeros:
                    if level == 1:
                        if topalike and (page[0] & 0x0000FFFFFFFFF000) in pt_addr_list:
                            pt_list[pa] = 1
                    elif count >= mincount:
                        pt_list[pa] = 1
                pa += 0x1000
        return pt_list

    def dump_dram(self, filename, pa, end_pa, buffer_size=0x100000):
        with open(filename, 'wb') as dram:
            self.logger.log(f'[*] Dumping memory to {filename} ...')
            while pa < end_pa:
                dram.write(self.cs.mem.read_physical_mem(pa, min(end_pa - pa, buffer_size)))
                pa += buffer_size
        return

    def run(self, module_argv):
        self.logger.start_test('EPT Finder')

        self.read_from_file = (len(module_argv) > 0) and (module_argv[0] == 'file')

        if self.read_from_file:
            if len(module_argv) == 3:
                revision_id = int(module_argv[2], 16)
                pattern = f'{module_argv[1]}.dram_*'
                filenames = glob.glob(pattern)
                for name in filenames:
                    addr = name[len(pattern) - 1:]
                    addr = 0 if addr == 'lo' else 0x100000000 if addr == 'hi' else int(addr, 16)
                    size = os.stat(name).st_size
                    self.logger.log(f'  Mapping file to address: 0x{addr:012x}  size: 0x{size:012x}  name: {name}')
                    self.par.append((addr, addr + size, open(name, 'rb')))
            else:
                self.logger.log_error('Invalid parameters')
                self.logger.log(self.__doc__.replace('`', ''))
                return ModuleResult.ERROR
        else:
            revision_id = self.cs.msr.read_msr(0, 0x480)[0]
            self.par = self.get_memory_ranges()

        if len(self.par) == 0:
            self.logger.log_error('Memory ranges are not defined!')
            return ModuleResult.ERROR

        if (len(module_argv) == 2) and (module_argv[0] == 'dump'):
            for (pa, end_pa, _) in self.par:
                postfix = 'lo' if pa == 0x0 else 'hi' if pa == 0x100000000 else f'0x{pa:08x}'
                filename = f'{module_argv[1]}.dram_{postfix}'
                self.dump_dram(filename, pa, end_pa)
            return ModuleResult.PASSED

        self.logger.log('[*] Searching Extended Page Tables ...')
        ept_pt_list = self.find_ept_pt({}, 0, 4)
        self.logger.log(f'[*] Found PTs  : {len(ept_pt_list):d}')
        ept_pd_list = self.find_ept_pt(ept_pt_list, 4, 3)
        self.logger.log(f'[*] Found PDs  : {len(ept_pd_list):d}')
        ept_pdpt_list = self.find_ept_pt(ept_pd_list, 1, 2)
        self.logger.log(f'[*] Found PDPTs: {len(ept_pdpt_list):d}')
        ept_pml4_list = self.find_ept_pt(ept_pdpt_list, 1, 1)
        self.logger.log(f'[*] Found PML4s: {len(ept_pml4_list):d}')
        self.logger.log('[*] -> EPTP: ' + ' '.join([f'{x:08X}' for x in sorted(ept_pml4_list.keys())]))
        ept_vmcs_list = self.find_vmcs_by_ept([x for x in ept_pml4_list.keys()], revision_id)
        self.logger.log(f'[*] Found VMCSs: {len(ept_vmcs_list):d}')
        self.logger.log('[*] -> VMCS: ' + ' '.join([f'{x:08X}' for x in sorted(ept_vmcs_list)]))

        try:
            self.path = 'VMs\\'
            os.makedirs(self.path)
        except OSError:
            pass

        for addr in sorted(ept_vmcs_list):
            write_file(self.path + f'vmcs_{addr:08x}.bin', self.read_physical_mem(addr))

        count = 1
        for eptp in sorted(ept_pml4_list.keys()):
            ept = c_extended_page_tables_from_file(self.cs, self.read_from_file, self.par)
            ept.prompt = f'[VM{count:d}]'
            ept.read_pt_and_show_status(self.path + f'ept_{eptp:08x}', 'Extended', eptp)
            if not ept.failure:
                ept.save_configuration(self.path + f'ept_{eptp:08x}.py')
            count += 1

        for (_, _, source_fp) in self.par:
            if source_fp:
                source_fp.close()

        return ModuleResult.INFORMATION



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
Xen hypercall fuzzer

Usage:
    ``chipsec_main.py -i -m tools.vmm.xen.hypercallfuzz -a <mode>[,<vector>,<iterations>]``

    - ``mode``                       : fuzzing mode

        * ``help``                 : Prints this help
        * ``info``                 : Hypervisor information
        * ``fuzzing``              : Fuzzing specified hypercall
        * ``fuzzing-all``          : Fuzzing all hypercalls
        * ``fuzzing-all-randomly`` : Fuzzing random hypercalls
    - ``<vector>``                 : Code or name of a hypercall to be fuzzed (use info)
    - ``<iterations>``             : Number of fuzzing iterations

Examples:
    >>> chipsec_main.py -i -m tools.vmm.xen.hypercallfuzz -a fuzzing,10 -l log.txt
    >>> chipsec_main.py -i -m tools.vmm.xen.hypercallfuzz -a fuzzing-all,50 -l log.txt
    >>> chipsec_main.py -i -m tools.vmm.xen.hypercallfuzz -a fuzzing-all-randomly,10,0x10000000 -l log.txt

.. note::
    - Returns a Warning by default
    - System may be in an unknown state, further evaluation may be needed

.. important::
    - This module is designed to run in a VM environment
    - Behavior on physical HW is undefined

"""

from chipsec.modules.tools.vmm.xen.define import *
from chipsec.module_common import BaseModule, CPU
from chipsec.library.returncode import ModuleResult
from chipsec.modules.tools.vmm.xen.hypercall import XenHypercall

TAGS = [CPU]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'TOOLS', 'VMM', 'XEN', 'HYPERCALLFUZZ']


class HypercallFuzz(BaseModule):
    def __init__(self):
        BaseModule.__init__(self)

    def usage(self):
        self.logger.log(self.__doc__.replace('`', ''))
        return

    def get_int(self, arg, base=10, defvalue=10000):
        try:
            value = int(arg, base)
        except ValueError:
            self.logger.log_error(f'Invalid integer parameter: \'{arg}\' (using default value: {defvalue:d})')
            value = defvalue
        return value

    def run(self, module_argv):
        self.logger.start_test('Xen Hypervisor Hypercall Fuzzer')
        command = module_argv[0] if len(module_argv) > 0 else ''
        arg1 = module_argv[1] if len(module_argv) > 1 else ''
        arg2 = module_argv[2] if len(module_argv) > 2 else ''

        xen = XenHypercall()
        xen.prompt = 'CHIPSEC'
        xen.debug = False

        if command == 'help':
            self.usage()
        elif command == 'info':
            info = xen.get_hypervisor_info()
            if len(info) > 0:
                xen.hypervisor_present = True
                xen.print_hypervisor_info(info)
                xen.scan_hypercalls(range(256))
                xen.print_hypercall_status()

        elif command == 'fuzzing':
            name2code = {v.lower(): k for k, v in hypercall_names.items()}
            try:
                code = int(arg1, 16)
            except ValueError:
                if arg1.lower() not in name2code:
                    self.logger.log_error(f'Unknown hypercall: \'{arg1}\'')
                    self.result.setStatusBit(self.result.status.UNSUPPORTED_OPTION)
                    return self.result.getReturnCode(ModuleResult.ERROR)
                code = name2code[arg1.lower()]
            count = self.get_int(arg2)
            xen.fuzz_hypercall(code, count)

        elif command in ['fuzzing-all', 'fuzzing-all-randomly']:
            count = self.get_int(arg1)
            xen.scan_hypercalls(range(256))
            xen.print_hypercall_status()
            self.logger.log('\nStart fuzzing ...\n')
            excluded = [MEMORY_OP, CONSOLE_IO, GRANT_TABLE_OP, SCHED_OP]
            vectors = sorted([x for x in xen.hypercalls.keys() if x not in excluded])
            if command == 'fuzzing-all':
                for vector in vectors:
                    xen.fuzz_hypercall(vector, count)
            else:
                xen.fuzz_hypercalls_randomly(vectors, count)
        else:
            self.logger.log(f'Invalid command: {command}\n')
            self.usage()

        self.logger.log_information('Module completed')
        self.logger.log_warning('System may be in an unknown state, further evaluation may be needed.')
        self.result.setStatusBit(self.result.status.POTENTIALLY_VULNERABLE)
        self.res = self.result.getReturnCode(ModuleResult.WARNING)
        return self.res



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#


"""
The uefi command provides access to UEFI variables, both on the live system and in a SPI flash image file.

>>> chipsec_util uefi types
>>> chipsec_util uefi var-list
>>> chipsec_util uefi var-list-spi [rom_file]
>>> chipsec_util uefi var-find <name>|<GUID>
>>> chipsec_util uefi var-read|var-write|var-delete <name> <GUID> <efi_variable_file>
>>> chipsec_util uefi decode <rom_file> [filetypes]
>>> chipsec_util uefi nvram[-auth] <rom_file> [fwtype]
>>> chipsec_util uefi keys <keyvar_file>
>>> chipsec_util uefi tables
>>> chipsec_util uefi s3bootscript [script_address]
>>> chipsec_util uefi assemble <GUID> freeform none|lzma|tiano <raw_file> <uefi_file>
>>> chipsec_util uefi insert_before|insert_after|replace|remove <GUID> <rom> <new_rom> <uefi_file>

Examples:

>>> chipsec_util uefi types
>>> chipsec_util uefi var-list
>>> chipsec_util uefi var-list-spi
>>> chipsec_util uefi var-find PK
>>> chipsec_util uefi var-read db D719B2CB-3D3A-4596-A3BC-DAD00E67656F db.bin
>>> chipsec_util uefi var-write db D719B2CB-3D3A-4596-A3BC-DAD00E67656F db.bin
>>> chipsec_util uefi var-delete db D719B2CB-3D3A-4596-A3BC-DAD00E67656F
>>> chipsec_util uefi decode uefi.rom
>>> chipsec_util uefi decode uefi.rom FV_MM
>>> chipsec_util uefi nvram uefi.rom vss_auth
>>> chipsec_util uefi keys db.bin
>>> chipsec_util uefi tables
>>> chipsec_util uefi s3bootscript
>>> chipsec_util uefi assemble AAAAAAAA-BBBB-CCCC-DDDD-EEEEEEEEEEEE freeform lzma uefi.raw mydriver.efi
>>> chipsec_util uefi replace  AAAAAAAA-BBBB-CCCC-DDDD-EEEEEEEEEEEE bios.bin new_bios.bin mydriver.efi
"""

import os
import uuid
from argparse import ArgumentParser

from chipsec.command import BaseCommand, toLoad
from chipsec.hal.uefi_common import EFI_STATUS_DICT, parse_efivar_file
from chipsec.library.file import write_file, read_file
from chipsec.hal.spi_uefi import decode_uefi_region, modify_uefi_region, compress_image, CMD_UEFI_FILE_REPLACE
from chipsec.hal.spi_uefi import CMD_UEFI_FILE_INSERT_AFTER, CMD_UEFI_FILE_INSERT_BEFORE, CMD_UEFI_FILE_REMOVE
from chipsec.hal.uefi import UEFI, decode_EFI_variables, get_attr_string, identify_EFI_NVRAM
from chipsec.hal.uefi import SECURE_BOOT_KEY_VARIABLES, parse_script, parse_EFI_variables
from chipsec.hal.uefi_fv import get_guid_bin, assemble_uefi_file, assemble_uefi_section, assemble_uefi_raw
from chipsec.hal.uefi_fv import FILE_TYPE_NAMES
from chipsec.hal.uefi_platform import fw_types


# Unified Extensible Firmware Interface (UEFI)
class UEFICommand(BaseCommand):

    def requirements(self) -> toLoad:
        if 'decode' in self.argv:
            return toLoad.Nil
        elif 'var-list-spi' in self.argv:
            return toLoad.All
        return toLoad.Driver

    def parse_arguments(self) -> None:
        parser = ArgumentParser(prog='chipsec_util uefi', usage=__doc__)
        subparsers = parser.add_subparsers()

        # var-read command args
        parser_var_read = subparsers.add_parser('var-read')
        parser_var_read.add_argument('name', type=str, help='name of variable to read')
        parser_var_read.add_argument('guid', type=str, help='guid of variable to read')
        parser_var_read.add_argument('filename', type=str, nargs='?', default=None, help='output file to store read variable contents to')
        parser_var_read.set_defaults(func=self.var_read)

        # var-write command args
        parser_var_write = subparsers.add_parser('var-write')
        parser_var_write.add_argument('name', type=str, help='name of variable to write')
        parser_var_write.add_argument('guid', type=str, help='guid of variable to write')
        parser_var_write.add_argument('filename', type=str, help='input file containing data to write to variable')
        parser_var_write.set_defaults(func=self.var_write)

        # var-delete command args
        parser_var_delete = subparsers.add_parser('var-delete')
        parser_var_delete.add_argument('name', type=str, help='name of variable to delete')
        parser_var_delete.add_argument('guid', type=str, help='guid of variable to delete')
        parser_var_delete.set_defaults(func=self.var_delete)

        # var-list command args
        parser_var_list = subparsers.add_parser('var-list')
        parser_var_list.set_defaults(func=self.var_list)

        # var-list-spi command args
        parser_var_read = subparsers.add_parser('var-list-spi')
        parser_var_read.add_argument('filename', type=str, nargs='?', default=None, help='Binary file of dumped rom if you have it')
        parser_var_read.set_defaults(func=self.var_list_spi)

        # var-find command args
        parser_var_find = subparsers.add_parser('var-find')
        parser_var_find.add_argument('name_guid', type=str, help='name or guid of variable to find')
        parser_var_find.set_defaults(func=self.var_find)

        # nvram command args
        parser_nvram = subparsers.add_parser('nvram')
        parser_nvram.add_argument('romfilename', type=str, help='nvram image')
        parser_nvram.add_argument('fwtype', type=str, nargs='?', default=None)
        parser_nvram.set_defaults(func=self.nvram)

        # nvram-auth command args
        parser_nvram_auth = subparsers.add_parser('nvram-auth')
        parser_nvram_auth.add_argument('romfilename', type=str, help='nvram image')
        parser_nvram_auth.add_argument('fwtype', type=str, nargs='?', default=None)
        parser_nvram_auth.set_defaults(func=self.nvram_auth)

        # decode command args
        parser_decode = subparsers.add_parser('decode')
        parser_decode.add_argument('filename', type=str, help='bios image to decompress')
        parser_decode.add_argument('--fwtype', dest='fwtype', type=str, nargs='?', default=None)
        parser_decode.add_argument('filetypes', type=str, nargs='*', default=[], help=FILE_TYPE_NAMES.values())
        parser_decode.set_defaults(func=self.decode)

        # keys command args
        parser_keys = subparsers.add_parser('keys')
        parser_keys.add_argument('filename', type=str, help='name of file containing variables')
        parser_keys.set_defaults(func=self.keys)

        # tables command args
        parser_tables = subparsers.add_parser('tables')
        parser_tables.set_defaults(func=self.tables)

        # s3bootscript command args
        parser_bootscript = subparsers.add_parser('s3bootscript')
        parser_bootscript.set_defaults(func=self.s3bootscript)
        parser_bootscript.add_argument('bootscript_pa', type=lambda x: int(x, 0), nargs='?', help='')

        # insert-before command args
        parser_insert_before = subparsers.add_parser('insert-before')
        parser_insert_before.add_argument('guid', type=str, help='guid')
        parser_insert_before.add_argument('filename', type=str, help='')
        parser_insert_before.add_argument('new_file', type=str, help='')
        parser_insert_before.add_argument('efi_file', type=str, help='')
        parser_insert_before.set_defaults(func=self.insert_before)

        # insert-after command args
        parser_insert_after = subparsers.add_parser('insert-after')
        parser_insert_after.add_argument('guid', type=str, help='guid')
        parser_insert_after.add_argument('filename', type=str, help='')
        parser_insert_after.add_argument('new_file', type=str, help='')
        parser_insert_after.add_argument('efi_file', type=str, help='')
        parser_insert_after.set_defaults(func=self.insert_after)

        # replace command args
        parser_replace = subparsers.add_parser('replace')
        parser_replace.add_argument('guid', type=str, help='guid')
        parser_replace.add_argument('filename', type=str, help='')
        parser_replace.add_argument('new_file', type=str, help='')
        parser_replace.add_argument('efi_file', type=str, help='')
        parser_replace.set_defaults(func=self.replace)

        # remove command args
        parser_remove = subparsers.add_parser('remove')
        parser_remove.add_argument('guid', type=str, help='guid')
        parser_remove.add_argument('filename', type=str, help='')
        parser_remove.add_argument('new_file', type=str, help='')
        parser_remove.set_defaults(func=self.remove)

        # assemble command args
        parser_assemble = subparsers.add_parser('assemble')
        parser_assemble.add_argument('guid', type=str, help='guid')
        parser_assemble.add_argument('file_type', type=str, help='')
        parser_assemble.add_argument('comp', type=str, help='')
        parser_assemble.add_argument('raw_file', type=str, help='')
        parser_assemble.add_argument('efi_file', type=str, help='')
        parser_assemble.set_defaults(func=self.assemble)

        parser.parse_args(self.argv, namespace=self)

    def set_up(self) -> None:
        self._uefi = UEFI(self.cs)

    def var_read(self):
        self.logger.log("[CHIPSEC] Reading EFI variable Name='{}' GUID={{{}}} to '{}' via Variable API..".format(self.name, self.guid, self.filename))
        var = self._uefi.get_EFI_variable(self.name, self.guid, self.filename)

    def var_write(self):
        self.logger.log("[CHIPSEC] writing EFI variable Name='{}' GUID={{{}}} from '{}' via Variable API..".format(self.name, self.guid, self.filename))
        status = self._uefi.set_EFI_variable_from_file(self.name, self.guid, self.filename)
        self.logger.log("[CHIPSEC] status: {}".format(EFI_STATUS_DICT[status]))
        if status == 0:
            self.logger.log("[CHIPSEC] writing EFI variable was successful")
        else:
            self.logger.log_error("writing EFI variable failed")

    def var_delete(self):
        self.logger.log("[CHIPSEC] Deleting EFI variable Name='{}' GUID={{{}}} via Variable API..".format(self.name, self.guid))
        status = self._uefi.delete_EFI_variable(self.name, self.guid)
        self.logger.log("Returned {}".format(EFI_STATUS_DICT[status]))
        if status == 0:
            self.logger.log("[CHIPSEC] deleting EFI variable was successful")
        else:
            self.logger.log_error("deleting EFI variable failed")

    def var_list(self):
        self.logger.log("[CHIPSEC] Enumerating all EFI variables via OS specific EFI Variable API..")
        efi_vars = self._uefi.list_EFI_variables()
        if efi_vars is None:
            self.logger.log_important("[CHIPSEC] Could not enumerate EFI Variables. You can try using the `var-list-spi` subcommand. Exit..")
            return
        self.logger.log("[CHIPSEC] Decoding EFI Variables..")
        _orig_logname = self.logger.LOG_FILE_NAME
        self.logger.set_log_file('efi_variables.lst', False)
        nvram_pth = 'efi_variables.dir'
        if not os.path.exists(nvram_pth):
            os.makedirs(nvram_pth)
        decode_EFI_variables(efi_vars, nvram_pth)
        self.logger.set_log_file(_orig_logname)
        self.logger.log("[CHIPSEC] Variables are in efi_variables.lst log and efi_variables.dir directory")

    def var_list_spi(self):
        self.logger.log("[CHIPSEC] Enumerating all EFI variables via OS specific EFI Variable API..")
        efi_vars = self._uefi.list_EFI_variables_spi(self.filename)
        if efi_vars is None:
            self.logger.log("[CHIPSEC] Could not enumerate EFI Variables (Legacy OS?). Exit..")
            return
        self.logger.log("[CHIPSEC] Decoding EFI Variables..")
        _orig_logname = self.logger.LOG_FILE_NAME
        self.logger.set_log_file('efi_variables.lst', False)
        nvram_pth = 'efi_variables.dir'
        if not os.path.exists(nvram_pth):
            os.makedirs(nvram_pth)
        decode_EFI_variables(efi_vars, nvram_pth)
        self.logger.set_log_file(_orig_logname)
        self.logger.log("[CHIPSEC] Variables are in efi_variables.lst log and efi_variables.dir directory")

    def var_find(self):
        _vars = self._uefi.list_EFI_variables()
        if _vars is None:
            self.logger.log_warning('Could not enumerate UEFI variables (non-UEFI OS?)')
            return
        is_guid = 0
        try:
            _input_var = str(uuid.UUID(self.name_guid))
            is_guid = 1
        except ValueError:
            _input_var = self.name_guid

        if is_guid:
            self.logger.log("[*] Searching for UEFI variable with GUID {{{}}}..".format(_input_var))
            for name in _vars:
                n = 0
                for (off, buf, hdr, data, guid, attrs) in _vars[name]:
                    if _input_var == guid:
                        var_fname = '{}_{}_{}_{:d}.bin'.format(name, guid, get_attr_string(attrs).strip(), n)
                        self.logger.log_good("Found UEFI variable {}:{}. Dumped to '{}'".format(guid, name, var_fname))
                        write_file(var_fname, data)
                    n += 1
        else:
            self.logger.log("[*] Searching for UEFI variable with name {}..".format(_input_var))
            name = _input_var
            if name in list(_vars.keys()):
                n = 0
                for (off, buf, hdr, data, guid, attrs) in _vars[name]:
                    var_fname = '{}_{}_{}_{:d}.bin'.format(name, guid, get_attr_string(attrs).strip(), n)
                    self.logger.log_good("Found UEFI variable {}:{}. Dumped to '{}'".format(guid, name, var_fname))
                    write_file(var_fname, data)
                    n += 1

    def nvram(self):
        authvars = 0
        rom = read_file(self.romfilename)
        if self.fwtype is None:
            self.fwtype = identify_EFI_NVRAM(rom)
            if self.fwtype is None:
                self.logger.log_error("Could not automatically identify EFI NVRAM type")
                return
        elif self.fwtype not in fw_types:
            self.logger.log_error("Unrecognized EFI NVRAM type '{}'".format(self.fwtype))
            return

        _orig_logname = self.logger.LOG_FILE_NAME
        self.logger.set_log_file((self.romfilename + '.nv.lst'), False)
        parse_EFI_variables(self.romfilename, rom, authvars, self.fwtype)
        self.logger.set_log_file(_orig_logname)

    def nvram_auth(self):
        authvars = 1
        rom = read_file(self.romfilename)
        if self.fwtype is None:
            self.fwtype = identify_EFI_NVRAM(rom)
            if self.fwtype is None:
                self.logger.log_error("Could not automatically identify EFI NVRAM type")
                return
        elif self.fwtype not in fw_types:
            self.logger.log_error("Unrecognized EFI NVRAM type '{}'".format(self.fwtype))
            return

        _orig_logname = self.logger.LOG_FILE_NAME
        self.logger.set_log_file((self.romfilename + '.nv.lst'), False)
        parse_EFI_variables(self.romfilename, rom, authvars, self.fwtype)
        self.logger.set_log_file(_orig_logname)

    def decode(self):
        if not os.path.exists(self.filename):
            self.logger.log_error("Could not find file '{}'".format(self.filename))
            return

        self.logger.log("[CHIPSEC] Parsing EFI volumes from '{}'..".format(self.filename))
        _orig_logname = self.logger.LOG_FILE_NAME
        self.logger.set_log_file(self.filename + '.UEFI.lst', False)
        cur_dir = self.cs.os_helper.getcwd()
        ftypes = []
        inv_filetypes = {v: k for k, v in FILE_TYPE_NAMES.items()}
        if self.filetypes:
            for mtype in self.filetypes:
                if mtype in inv_filetypes.keys():
                    if inv_filetypes[mtype] not in ftypes:
                        ftypes.append(inv_filetypes[mtype])
                    break
        decode_uefi_region(cur_dir, self.filename, self.fwtype, ftypes)
        self.logger.set_log_file(_orig_logname)

    def keys(self):
        if not os.path.exists(self.filename):
            self.logger.log_error("Could not find file '{}'".format(self.filename))
            return
        self.logger.log("<keyvar_file> should contain one of the following EFI variables\n[ %s ]" % (" | ".join(["%s" % var for var in SECURE_BOOT_KEY_VARIABLES])))
        self.logger.log("[CHIPSEC] Parsing EFI variable from '{}'..".format(self.filename))
        parse_efivar_file(self.filename)

    def tables(self):
        self.logger.log("[CHIPSEC] Searching memory for and dumping EFI tables (this may take a minute)..\n")
        self._uefi.dump_EFI_tables()

    def s3bootscript(self):
        self.logger.log("[CHIPSEC] Searching for and parsing S3 resume bootscripts..")
        if self.bootscript_pa is not None:
            self.logger.log('[*] Reading S3 boot-script from memory at 0x{:016X}..'.format(self.bootscript_pa))
            script_all = self.cs.mem.read_physical_mem(self.bootscript_pa, 0x100000)
            self.logger.log('[*] Decoding S3 boot-script opcodes..')
            script_entries = parse_script(script_all, True)
        else:
            (bootscript_PAs, parsed_scripts) = self._uefi.get_s3_bootscript(True)

    def insert_before(self):
        if get_guid_bin(self.guid) == '':
            self.logger.log_bad('*** Error *** Invalid GUID: {}'.format(self.guid))
            return

        if not os.path.isfile(self.rom_file):
            self.logger.log_bad('*** Error *** File doesn\'t exist: {}'.format(self.rom_file))
            return

        if not os.path.isfile(self.efi_file):
            self.logger.log_bad('*** Error *** File doesn\'t exist: {}'.format(self.efi_file))
            return

        rom_image = read_file(self.rom_file)
        efi_image = read_file(self.efi_file)
        new_image = modify_uefi_region(rom_image, CMD_UEFI_FILE_INSERT_BEFORE, self.guid, efi_image)
        write_file(self.new_file, new_image)

    def insert_after(self):
        if get_guid_bin(self.guid) == '':
            self.logger.log_bad('*** Error *** Invalid GUID: {}'.format(self.guid))
            return

        if not os.path.isfile(self.rom_file):
            self.logger.log_bad('*** Error *** File doesn\'t exist: {}'.format(self.rom_file))
            return

        if not os.path.isfile(self.efi_file):
            self.logger.log_bad('*** Error *** File doesn\'t exist: {}'.format(self.efi_file))
            return

        rom_image = read_file(self.rom_file)
        efi_image = read_file(self.efi_file)
        new_image = modify_uefi_region(rom_image, CMD_UEFI_FILE_INSERT_AFTER, self.guid, efi_image)
        write_file(self.new_file, new_image)

    def replace(self):
        if get_guid_bin(self.guid) == '':
            self.logger.log_bad('*** Error *** Invalid GUID: {}'.format(self.guid))
            return

        if not os.path.isfile(self.rom_file):
            self.logger.log_bad('*** Error *** File doesn\'t exist: {}'.format(self.rom_file))
            return

        if not os.path.isfile(self.efi_file):
            self.logger.log_bad('*** Error *** File doesn\'t exist: {}'.format(self.efi_file))
            return

        rom_image = read_file(self.rom_file)
        efi_image = read_file(self.efi_file)
        new_image = modify_uefi_region(rom_image, CMD_UEFI_FILE_REPLACE, self.guid, efi_image)
        write_file(self.new_file, new_image)

    def remove(self):
        if get_guid_bin(self.guid) == '':
            self.logger.log_bad('*** Error *** Invalid GUID: {}'.format(self.guid))
            return

        if not os.path.isfile(self.rom_file):
            self.logger.log_bad('*** Error *** File doesn\'t exist: {}'.format(self.rom_file))
            return

        rom_image = read_file(self.rom_file)
        new_image = modify_uefi_region(rom_image, CMD_UEFI_FILE_REMOVE, self.guid)
        write_file(self.new_file, new_image)

    def assemble(self):
        compression = {'none': 0, 'tiano': 1, 'lzma': 2}

        if get_guid_bin(self.guid) == '':
            self.logger.log_bad('*** Error *** Invalid GUID: {}'.format(self.guid))
            return

        if not os.path.isfile(self.raw_file):
            self.logger.log_bad('*** Error *** File doesn\'t exist: {}'.format(self.raw_file))
            return

        if self.comp not in compression:
            self.logger.log_bad('*** Error *** Unknown compression: {}'.format(self.comp))
            return

        compression_type = compression[self.comp]

        if self.file_type == 'freeform':
            raw_image = read_file(self.raw_file)
            wrap_image = assemble_uefi_raw(raw_image)
            if compression_type > 0:
                comp_image = compress_image(wrap_image, compression_type)
                wrap_image = assemble_uefi_section(comp_image, len(wrap_image), compression_type)
            uefi_image = assemble_uefi_file(self.guid, wrap_image)
            write_file(self.efi_file, uefi_image)
        else:
            self.logger.log_bad('*** Error *** Unknow file type: {}'.format(self.file_type))
            return

        self.logger.log("[CHIPSEC]  UEFI file was successfully assembled! Binary file size: {:d}, compressed UEFI file size: {:d}".format(len(raw_image), len(uefi_image)))


commands = {'uefi': UEFICommand}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2019-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#

"""
>>> chipsec_util smbios entrypoint
>>> chipsec_util smbios get [raw|decoded] [type]

Examples:

>>> chipsec_util smbios entrypoint
>>> chipsec_util smbios get raw
"""

from argparse import ArgumentParser
from chipsec.command import BaseCommand, toLoad
from chipsec.hal.smbios import SMBIOS
from chipsec.library.logger import print_buffer_bytes
from chipsec.library.options import Options


class smbios_cmd(BaseCommand):

    def requirements(self) -> toLoad:
        return toLoad.All

    def parse_arguments(self) -> None:
        options = Options()
        try:
            default_type = options.get_section_data('Util_Config', 'smbios_get_type')
        except Exception:
            default_type = 'raw'

        parser = ArgumentParser(prog='chipsec_util smbios', usage=__doc__)
        subparsers = parser.add_subparsers()
        parser_entrypoint = subparsers.add_parser('entrypoint')
        parser_entrypoint.set_defaults(func=self.smbios_ep)
        parser_get = subparsers.add_parser('get')
        parser_get.add_argument('method', choices=['raw', 'decoded'], default=default_type, nargs='?',
                                help='Get raw data or decoded data.  Decoded data may not exist for all structures')
        parser_get.add_argument('type', type=int, default=None, nargs='?',
                                help='SMBIOS type to search for')
        parser_get.add_argument('-f', '--force', action='store_true', dest='_force_32',
                                help='Force reading from 32bit structures')
        parser_get.set_defaults(func=self.smbios_get)
        parser.parse_args(self.argv, namespace=self)

    def smbios_ep(self):
        self.logger.log('[CHIPSEC] SMBIOS Entry Point Structures')
        if self.smbios.smbios_2_pa is not None:
            self.logger.log(self.smbios.smbios_2_ep)
        if self.smbios.smbios_3_pa is not None:
            self.logger.log(self.smbios.smbios_3_ep)

    def smbios_get(self):
        if self.method == 'raw':
            self.logger.log('[CHIPSEC] Dumping all requested structures in raw format')
            structs = self.smbios.get_raw_structs(self.type, self._force_32)
        elif self.method == 'decoded':
            self.logger.log('[CHIPSEC] Dumping all requested structures in decoded format')
            structs = self.smbios.get_decoded_structs(self.type, self._force_32)
        if structs is None:
            self.logger.log('[CHIPSEC] Error getting data')
            return
        if len(structs) == 0:
            self.logger.log('[CHIPSEC] Structures not found')
            return

        for data in structs:
            if self.method == 'raw':
                header = self.smbios.get_header(data)
                if header is not None:
                    self.logger.log(header)
                self.logger.log('[CHIPSEC] Raw Data')
                print_buffer_bytes(data)
            elif self.method == 'decoded':
                self.logger.log(data)
            self.logger.log('==================================================================')

    def run(self):
        # Create and initialize SMBIOS object for commands to use
        try:
            self.logger.log('[CHIPSEC] Attempting to detect SMBIOS structures')
            self.smbios = SMBIOS(self.cs)
            found = self.smbios.find_smbios_table()
            if not found:
                self.logger.log('[CHIPSEC] Unable to detect SMBIOS structure(s)')
                return
        except Exception as e:
            self.logger.log(e)
            return

        self.func()


commands = {'smbios': smbios_cmd}



# CHIPSEC: Platform Security Assessment Framework
# Copyright (c) 2010-2021, Intel Corporation
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# Contact information:
# chipsec@intel.com
#
#
# Authors:
#  Yuriy Bulygin
#  Erik Bjorge
#


"""
SPI Flash Region Access Control

Checks SPI Flash Region Access Permissions programmed in the Flash Descriptor

Usage:
    ``chipsec_main -m common.spi_access``

Examples:
    >>> chipsec_main.py -m common.spi_access

Registers used:
    - HSFS.FDV
    - FRAP.BRWA

.. important::
    - Some platforms may use alternate means of protecting these regions.
      Consider this when assessing results.

"""

from chipsec.library.exceptions import CSReadError
from chipsec.module_common import BaseModule, BIOS
from chipsec.library.returncode import ModuleResult
from chipsec.hal.spi import SPI, GBE, PLATFORM_DATA, ME, FLASH_DESCRIPTOR
from typing import List

TAGS = [BIOS]
METADATA_TAGS = ['OPENSOURCE', 'IA', 'COMMON', 'SPI_ACCESS']


class spi_access(BaseModule):

    def __init__(self):
        BaseModule.__init__(self)

    def is_supported(self) -> bool:
        if self.cs.register.has_field('HSFS', 'FDV') and self.cs.register.has_field('FRAP', 'BRWA'):
            return True
        self.logger.log_important('HSFS.FDV or FRAP.BRWA registers not defined for platform.  Skipping module.')
        return False

    def check_flash_access_permissions(self) -> int:

        res = ModuleResult.PASSED
        fdv = self.cs.register.read_field('HSFS', 'FDV') == 1
        frap = self.cs.register.read('FRAP')
        brwa = self.cs.register.get_field('FRAP', frap, 'BRWA')

        if not fdv:
            self.logger.log('[*] Flash Descriptor Valid bit is not set')

        if brwa & (1 << PLATFORM_DATA):
            self.logger.log('[*] Software has write access to Platform Data region in SPI flash (platform specific)')

        if brwa & (1 << GBE):
            res = ModuleResult.WARNING
            self.result.setStatusBit(self.result.status.ACCESS_RW)
            self.logger.log_warning('Software has write access to GBe region in SPI flash')

        if brwa & (1 << FLASH_DESCRIPTOR):
            res = ModuleResult.FAILED
            self.result.setStatusBit(self.result.status.ACCESS_RW)
            self.logger.log_bad('Software has write access to SPI flash descriptor')

        if brwa & (1 << ME):
            res = ModuleResult.FAILED
            self.result.setStatusBit(self.result.status.ACCESS_RW)
            self.logger.log_bad('Software has write access to Management Engine (ME) region in SPI flash')

        if fdv:
            if ModuleResult.PASSED == res:
                self.logger.log_passed('SPI Flash Region Access Permissions in flash descriptor look ok')
            elif ModuleResult.FAILED == res:
                self.logger.log_failed('SPI Flash Region Access Permissions are not programmed securely in flash descriptor')
                self.logger.log_important('System may be using alternative protection by including descriptor region in SPI Protected Range Registers')
                self.logger.log_important('If using alternative protections, this can be considered a WARNING')
            elif ModuleResult.WARNING == res:
                self.logger.log_warning('Certain SPI flash regions are writeable by software')
        else:
            res = ModuleResult.WARNING
            self.result.setStatusBit(self.result.status.UNSUPPORTED_FEATURE)
            self.logger.log_warning('Either flash descriptor is not valid or not present on this system')

        return self.result.getReturnCode(res)

    def run(self, module_argv: List[str]) -> int:
        self.logger.start_test('SPI Flash Region Access Control')
        try:
            self.spi = SPI(self.cs)
            self.spi.display_SPI_Ranges_Access_Permissions()
            self.res = self.check_flash_access_permissions()
        except CSReadError as err:
            self.logger.log_warning(f'Unable to read register: {err}')
            self.result.setStatusBit(self.result.status.VERIFY)
            self.res = self.result.getReturnCode(ModuleResult.WARNING)
        return self.res
